package edu.utk.cs.icl.perfometer.datatransfer;

import edu.utk.cs.icl.perfometer.event.PerfBreachEvent;
import edu.utk.cs.icl.perfometer.event.BreachListener;

import edu.utk.cs.icl.perfometer.metrics.MetricParser;
import edu.utk.cs.icl.perfometer.metrics.UnsupportedMetricException;

//import java.util.LinkedList;
import com.sun.java.util.collections.*;

import java.util.Collections;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

import java.net.Socket;
import java.net.UnknownHostException;

import java.io.IOException;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;

public class LineReader extends Thread implements DataReader {
  // Variables required for communication with the server
  protected boolean isServer = false;
  protected boolean needApp = false;
  protected boolean stop = false;

  protected MetricParser parser;

  protected PerfHwInfo hwinfo;
  protected PerfProgramInfo exeinfo;
  protected PerfMetric[] eventList;

  protected Socket connection0;
  protected Socket connection1;
  protected DataInputStream socket0in;
  protected DataOutputStream socket0out;
  protected DataOutputStream socket1out;
  
  protected String spaces = "                                                                 ";
  protected LinkedList recycleBin;
  protected LinkedList itemBuffer = new LinkedList();
    
  protected boolean paused = false;

  protected int linesize = 8192;
  protected byte[] curline = new byte[linesize];

  protected PerfMessage perfMsg = new PerfMessage(200);

  protected AppInput apps;

  protected ProcessManager procmgr;

  protected Vector breachListeners = new Vector();

  class PerfInitFinishedException extends Exception {
    public PerfInitFinishedException(String message) {
      super(message);
    }
  }

  class PerfInitFailedException extends Exception {
    public PerfInitFailedException(String message) {
      super(message);
    }
  }

  public LineReader(ProcessManager p, MetricParser m)
  {
    if(m == null)
      throw new IllegalArgumentException("No Metric Specified");

    if(p == null)
      throw new IllegalArgumentException("No Process Manager Specified");

    procmgr = p;
    parser = m;
    parser.reset();

    procmgr.setDataReader(this);

    recycleBin = new LinkedList();
    paused = true;
  }

  public void setAppInput(AppInput a) {
    apps = a;
  }

  public void establishConnection(String host, int port) 
    throws java.io.IOException
  {

    setupSockets(host, port);

    // After the socket is opened, initialize communications with
    // either an application or the server for multiple applications
    handshake(host);
  }

  public boolean isServer() {
    return isServer;
  }
    
  public MetricParser getCurrentMetric() {
    return parser;
  }

  public boolean readyToRead() throws IOException {
    if(socket0in == null)
      return false;

    return (socket0in.available() > 0);
  }

  public void run() { 
    String line; 

    paused = false;

    if(DEBUG_CONNECTION) 
      System.err.println("linereader: going to begin reading now..");

    while( true ) { 
      if(stop) return;

      if(itemBuffer.size() > 30) {
        try {
          if(DEBUG_CONNECTION) 
            System.err.println("linereader: full buffer, sleeping..");

          java.lang.Thread.sleep(100);
        } catch (InterruptedException t) {   
        }
      }

      line = null;

      //if(DEBUG_CONNECTION) 
      //  System.err.println("linereader: lets see if we're ready to read..");

      try {
        synchronized(this) {
          if(readyToRead())
          {
            if(DEBUG_CONNECTION) 
              System.err.println("linereader: going to read a message now..");
            line = readMessage(socket0in, perfMsg);
          }
          //this.notifyAll();
        }
      }catch(IOException e) {
        if(DEBUG_CONNECTION) 
          System.err.println("linereader: error reading a message:"+e.getMessage());
        return;
      }
            
      while (paused) {
        synchronized(this) {
          this.notifyAll();
        }
        if(stop) return;
        try {
          java.lang.Thread.sleep(200);
        } catch (InterruptedException t) {   
          if(stop) return;
        } catch (Throwable t2) {   
          if(DEBUG_CONNECTION) System.err.println(t2.getMessage());
          t2.printStackTrace();
          //Ensure the application exits with an error condition.
          System.exit(1);
        }
      }
 
      if(line != null) {
        if(DEBUG_CONNECTION) System.err.println("Read line: " + line);
        parseInput(line, perfMsg.type);
      }
      this.yield();
    }
  }

  void setupSockets(String host, int portNum) throws IOException {
    int currentPortNum = portNum;
    boolean gotSocket = false;

    do {
      try {
        connection0 = new Socket(host, currentPortNum);
        socket0out = new DataOutputStream(
           new BufferedOutputStream(connection0.getOutputStream()));
        socket0in = new DataInputStream(
           new BufferedInputStream(connection0.getInputStream()));
        gotSocket = true;
      } catch (UnknownHostException e) {
        throw new IOException("Can't find host " + host);
      } catch (IOException e) {
        if (currentPortNum < portNum + 5) {
          currentPortNum++;
          gotSocket = false;
        } else {
          throw e;
        }
      }
    } while (!gotSocket);
  }

  // This routine sets up the second socket that we use for output
  void setupSockets2(String host, int portNum) throws IOException {
    try {
      connection1 = new Socket(host, portNum);
      socket1out = new DataOutputStream(
         new BufferedOutputStream(connection1.getOutputStream()));
    } catch (UnknownHostException e) {
      throw new IOException("Can't find host " + host);
    }
  }
    
  // Get the info from the application about the hardware and executable
  int initInfo() throws IOException, 
    PerfInitFinishedException, PerfInitFailedException
  {
    String s = "";
    StringTokenizer st;
    
    if(DEBUG_CONNECTION) System.err.println("About to get hwinfo");    
    // First get the hardware info in the following format:
    s = readMessage(socket0in, perfMsg);


    if(perfMsg.type == PERF_ACK_int)
      throw new PerfInitFinishedException("done");
    else if(perfMsg.type == PERF_FAILEDCONN_int)
      throw new PerfInitFailedException("Server reported init failure.");

    st = new StringTokenizer(s, STR_DEL);
    int hostIdx = Integer.parseInt(st.nextToken()) - 1;
    if(DEBUG_CONNECTION) System.err.println("hostIdx = " + hostIdx);

    if(DEBUG_CONNECTION) System.err.println("Read line:" + s + "|");
    hwinfo = parseHwInfo(s);

    if(DEBUG_CONNECTION) System.err.println("About to get exeinfo");
    // Then get the executable info:
    s = readMessage(socket0in, perfMsg);
    if(DEBUG_CONNECTION) System.err.println("Read line:" + s + "|");
    exeinfo = parseProgramInfo(s);
        
    if(DEBUG_CONNECTION) System.err.println("About to get event info");
    // Now get the list of events
    s = readMessage(socket0in, perfMsg);
    if(DEBUG_CONNECTION) System.err.println("Read line:" + s + "|");
    eventList = parseEventInfo(s);
        
    if(DEBUG_CONNECTION) System.err.println("Sending ack");
    // Now send an ACK
    sendAck(socket1out);

    return hostIdx;
  }
    
  // Make sure we are talking to a perfometer client
  void handshake(String host) throws IOException {
    String serverIP;
    String userName;
    int secondPortNum = 0;
    String s;
    StringTokenizer st;

    if(DEBUG_CONNECTION) System.err.println("Entering handshake");
    s = readMessage(socket0in, perfMsg);
    isServer = (perfMsg.type == PERF_SERVER_int);

    if(DEBUG_CONNECTION) System.err.println("Read line 1:" + s + "|");

    st = new StringTokenizer(s, STR_DEL);

    userName = st.nextToken();
    serverIP = st.nextToken();
    s = st.nextToken();
    needApp = s.equals(PERF_NEEDAPP);

    if(!needApp)
      Integer.parseInt(st.nextToken());   // numgraphs

    if(DEBUG_CONNECTION)
       System.err.println("isServer: " + isServer + " User: " + 
         userName + " IP: " + serverIP + " NeedApp: " + needApp);

    // We should respond with PERF_ACK
    sendAck(socket0out);

    if(DEBUG_CONNECTION) System.err.println("waiting for second port num\n");
    // Now get the port number for the second socket to the server
    secondPortNum = getSecondPort(socket0in);
    
    // And open that second socket for writing
    if(DEBUG_CONNECTION) 
      System.err.println(" about to open second port: " + secondPortNum);

    setupSockets2(host, secondPortNum);
    if(DEBUG_CONNECTION) System.err.println("Socket opened sending ack on new port");
    sendAck(socket1out);

    // Listen for an ACK, to make sure the server can hear us on
    // the newly opened port
    if(DEBUG_CONNECTION) 
      System.err.println("ACK sent, listening for ACK from server");

    getMsg(socket0in, PERF_ACK_int);

    if(DEBUG_CONNECTION) System.err.println("ACK received");

    //MPI?
        
    // Now if the server needs the list of applications, get that
    // from the user and send it in this format:
    // ^App1host^App1port
    // ^App2host^App2port
    // ^PERF_END
    if (needApp) {
      String [] hosts;
      String hstring;

      if(DEBUG_CONNECTION) System.err.println("need to specify apps");

      if(apps == null)
        throw new IOException("AppInput not provided.");

      if(DEBUG_CONNECTION) System.err.println("waiting for hosts");
      apps.waitForHosts();
      hosts = apps.getHosts();
      if(DEBUG_CONNECTION) System.err.println("got hosts");

      for(int x = 0; x < hosts.length; x++) {
        st = new StringTokenizer(hosts[x]);        
        hstring = st.nextToken() + STR_DEL +
            st.nextToken();
        putLine(socket1out, PERF_APP_int, hstring);
      }

      if(DEBUG_CONNECTION) System.err.println("after sending apps..");

      putLine(socket1out, PERF_END_int, null);

    } else {
      if(DEBUG_CONNECTION) System.err.println("In NoNeedApp waiting");

      if(isServer ) {
        do {
          readMessage(socket0in, perfMsg);
        } while ( perfMsg.type != PERF_END_int );
      }
    }

    if(DEBUG_CONNECTION) System.err.println("going to wait for info now..");

    while(true) {
      try {
        // Get the info from the application about the hardware and executable
        int idx = initInfo();

        procmgr.add(idx, new PerfProcessInfo(idx, hwinfo, exeinfo, eventList), 
          parser.getNewInstance());
      } catch (PerfInitFailedException e) {
        throw new IOException(e.getMessage());
      } catch (PerfInitFinishedException e2) {
        break; 
      } catch (Exception e3) {
        if(DEBUG_CONNECTION) 
          System.err.println("Error inserting..." + e3.getMessage());

        throw new IOException("Error inserting.." + e3.getMessage());
      }

      // if this isn't a server, break after first iteration
      if(!isServer)
        break;
    }

    if(DEBUG_CONNECTION) System.err.println("finished handshaking...");
  }

  int getSecondPort(DataInputStream in) throws IOException {
    readMessage(in, perfMsg);

    return perfMsg.type;
  }

  String readMessage(DataInputStream inSocket, PerfMessage m) throws IOException {
    if(m == null)
      throw new IOException("Null message structure");

    if(DEBUG_CONNECTION) System.err.println("enter readMessage");

    m.len = inSocket.readInt();
    m.type = inSocket.readInt();  // ignore for now
 
    if(DEBUG_CONNECTION) 
      System.err.println("readMessage: len = " + m.len + ", type = " +m.type);

    if(m.len > m.msg.length)
      m.msg = new byte[m.len];

    inSocket.readFully(m.msg, 0, m.len);

    return new String(m.msg, 0, m.len);
  }

  void putLine(DataOutputStream d, int type, String s)
     throws IOException
  {
    if(s == null)
      d.writeInt(0);
    else
      d.writeInt(s.length() + STR_DEL.length());

    if(type >= 0)
      d.writeInt(type);

    if(DEBUG_CONNECTION) {
      if(s != null)
        System.err.println("sending (len = " + 
          (s.length() + STR_DEL.length()) + ", type = " + type +") : " + s);
      else
        System.err.println("sending (len = 0 type = " + type +") : " + s);
    }

    if(s != null)
      d.writeBytes(STR_DEL + s);

    d.flush();

    if(DEBUG_CONNECTION)
      System.err.println("line sent, leaving putLine..");
  }

  public synchronized void sendStart(int idx) {
    String msg = Integer.toString(idx+1);

    try {
      putLine(socket1out, PERF_STARTAPP_int, msg);
    } catch (IOException e) {
      if(DEBUG_CONNECTION) 
        System.err.println("error sending message: " + e.getMessage());
    }
  }

  public synchronized void sendChangeMetric(int idx, MetricParser m)
    throws UnsupportedMetricException
  {
    String [] metrics = m.getMetricNames();
    StringBuffer msgBuf = new StringBuffer(Integer.toString(idx+1) + STR_DEL);
    String msg, line;
    int i;

    if(DEBUG_CONNECTION)
      System.out.println("enter sendchangemetric..");

    for(i=0;i< metrics.length-1;i++)
      msgBuf.append(metrics[i] + RANK_DEL);
    msgBuf.append(metrics[i]);

    msg = msgBuf.toString();

    try {
      if(!paused) {
        this.paused = true;

        if(DEBUG_CONNECTION)
          System.out.println("waiting..");

        this.wait(10000);
      }

      if(DEBUG_CONNECTION) 
        System.out.println("going to send change metric to: " + m);

      putLine(socket1out, PERF_CHANGE_EVENT_int, msg);

      if(DEBUG_CONNECTION)
        System.out.println("now waiting for OK..");

      do {
        line = readMessage(socket0in, perfMsg);
      } while((perfMsg.type != PERF_OK_int) 
          && (perfMsg.type != PERF_ERROR_int));

      if(perfMsg.type == PERF_ERROR_int)
        throw new UnsupportedMetricException(new String(perfMsg.msg,0,perfMsg.len));

      if(DEBUG_CONNECTION) System.out.println("got OK..");

      parser = m;
      parser.reset();
      procmgr.changeAllMetrics(parser);

    } catch (IOException e) {
      if(DEBUG_CONNECTION)
        System.err.println("error sending message: " + e.getMessage());
    } catch (InterruptedException e2) {
      if(DEBUG_CONNECTION)
        System.err.println("Interrupted waiting for OK: " + e2.getMessage());
    }

    if(DEBUG_CONNECTION)
      System.out.println("done, un-pausing..");

    this.paused = false;
  }

  //hack!!
  public void sendChangeMetricUnsynchronized(int idx, MetricParser m) 
    throws UnsupportedMetricException
  {
    String [] metrics = m.getMetricNames();
    StringBuffer msgBuf = new StringBuffer(Integer.toString(idx+1) + STR_DEL);
    String msg, line;
    int i;

    for(i=0;i< metrics.length-1;i++)
      msgBuf.append(metrics[i] + RANK_DEL);
    msgBuf.append(metrics[i]);

    msg = msgBuf.toString();

    try {

      if(DEBUG_CONNECTION)
        System.out.println("going to send change metric to: " + m);

      putLine(socket1out, PERF_CHANGE_EVENT_int, msg);

      if(DEBUG_CONNECTION)
        System.out.println("now waiting for OK..");

      do {
        line = readMessage(socket0in, perfMsg);
      } while((perfMsg.type != PERF_OK_int)
          && (perfMsg.type != PERF_ERROR_int));

      if(perfMsg.type == PERF_ERROR_int)
        throw new UnsupportedMetricException(new String(perfMsg.msg,0,perfMsg.len));

      if(DEBUG_CONNECTION) System.out.println("got OK..");

      parser = m;
      parser.reset();
      procmgr.changeAllMetrics(parser);

    } catch (IOException e) {
      if(DEBUG_CONNECTION)
        System.err.println("error sending message: " + e.getMessage());
    }
  }


  public synchronized void sendPause(int idx) {
    String msg = Integer.toString(idx+1);

    //this.paused = true;

    try {
      putLine(socket1out, PERF_PAUSE_int, msg);
    } catch (IOException e) {
      if(DEBUG_CONNECTION) 
        System.err.println("error sending message: " + e.getMessage());
    }
  }

  public synchronized void sendPollingInterval(int idx, int pollIntMs) {
    String msg = (idx+1) + STR_DEL + pollIntMs;

    if(DEBUG_CONNECTION) System.out.println("sendPollingInterval (" +
       pollIntMs + ")");

    try {
      putLine(socket1out, PERF_POLL_int, msg);
    } catch (IOException e) {
      if(DEBUG_CONNECTION) 
        System.err.println("error sending message: " + e.getMessage());
    }
  }

  public synchronized void sendDatastreamStop(int idx) {
    String msg = (idx+1) + STR_DEL + "0";

    try {
      putLine(socket1out, PERF_DATASTREAM_int, msg);
    } catch (IOException e) {
      if(DEBUG_CONNECTION) 
        System.err.println("error sending message: " + e.getMessage());
    }
  }

  public synchronized void sendDatastreamStart(int idx) {
    String msg = (idx+1) + STR_DEL + "1";

    try {
      putLine(socket1out, PERF_DATASTREAM_int, msg);
    } catch (IOException e) {
      if(DEBUG_CONNECTION) 
        System.err.println("error sending message: " + e.getMessage());
    }
  }

  public synchronized void sendPopupParameters(int idx, 
    double min, double max, int thresh, int data)
  {
    String msg = (idx+1) + STR_DEL + min
        + STR_DEL + max + STR_DEL + thresh
        + STR_DEL + data;

    try {
      putLine(socket1out, PERF_POPUP_int, msg);
    } catch (IOException e) {
      if(DEBUG_CONNECTION) 
        System.err.println("error sending message: " + e.getMessage());
    }
  }

  void sendAck(DataOutputStream out) throws IOException {
    out.writeInt(0);
    out.writeInt(PERF_ACK_int);
    out.flush();
  }

  // getMsg reads a line of text from the socket, sock, and
  // checks to see if it is equal to msg
  void getMsg(DataInputStream sock, int msgtype) throws IOException {
    readMessage(sock, perfMsg);
    
    if (perfMsg.type != msgtype)
      throw new IOException("Unexpected message.");
  }

  public void disconnect() {
    stop = true;

    try {
      //socket0in.close();
      //socket0out.close();
      //socket1out.close();
      connection0.close();
      connection1.close();
    } catch(Exception e) {
    }
  }

  // Get the hardware info in the following format:
  // ^PERF_DATA^host_id^0^PERF_HWINFO^#cpus^#nodes^total cpus^vendor #^vendor string
  // ^model #^model string^revision^mhz
  PerfHwInfo parseHwInfo(String s) throws NoSuchElementException {
    StringTokenizer st = new StringTokenizer(s, STR_DEL);
    PerfHwInfo newhwinfo = new PerfHwInfo();

    s = st.nextToken(); // Throwing away host id for now
    s = st.nextToken();
    s = st.nextToken();
    if (!s.equals(PERF_HWINFO))
      throw new NoSuchElementException("error");
    newhwinfo.machinename = st.nextToken();
    newhwinfo.ncpu = Integer.parseInt(st.nextToken());
    newhwinfo.nnodes = Integer.parseInt(st.nextToken());
    newhwinfo.totalcpus = Integer.parseInt(st.nextToken());
    newhwinfo.vendor = Integer.parseInt(st.nextToken());
    newhwinfo.vendor_string = st.nextToken();
    newhwinfo.model = Integer.parseInt(st.nextToken());
    newhwinfo.model_string = st.nextToken();
    newhwinfo.revision = Float.valueOf(st.nextToken()).floatValue();
    newhwinfo.mhz = Float.valueOf(st.nextToken()).floatValue();

    return newhwinfo;
  }

  // Then get the executable info:
  // ^PERF_DATA^host_id^0^PERF_EXEINFO^executable fullname
  // ^executable name^pre-loaded environment
  PerfProgramInfo parseProgramInfo(String s) {
    StringTokenizer st = new StringTokenizer(s, STR_DEL);
    PerfProgramInfo newexeinfo = new PerfProgramInfo();

    s = st.nextToken(); // Throwing away host id for now
    s = st.nextToken();
    s = st.nextToken();
    if (!s.equals(PERF_EXEINFO))
      throw new NoSuchElementException("error");

    newexeinfo.fullname = st.nextToken();
    newexeinfo.name = st.nextToken();
    newexeinfo.lib_preload_env = st.nextToken();

    return newexeinfo;
  }

  // Now get the list of events in this format:
  // ^PERF_DATA^my hostip^0^PERF_EVENTLIST^event1name^event1desc^event2name^event2desc\0
  PerfMetric[] parseEventInfo(String s) {
    StringTokenizer st = new StringTokenizer(s, STR_DEL);
    PerfMetric[] neweventList = new PerfMetric[300];

    if(DEBUG_CONNECTION) System.err.println("parsing event info..");

    s = st.nextToken(); // Throwing away host id for now
    if(DEBUG_CONNECTION) System.err.println("s = " + s);

    s = st.nextToken();
    if(DEBUG_CONNECTION) System.err.println("s = " + s);

    s = st.nextToken();
    if(DEBUG_CONNECTION) System.err.println("s = " + s);

    if (!s.equals(PERF_EVENTLIST))
      throw new NoSuchElementException("error");
      
    int counter = 0;
    String tempstr;
    while(st.hasMoreTokens()) {
      tempstr = st.nextToken();
      neweventList[counter] = new PerfMetric(tempstr, st.nextToken());
      counter++;
    }

    if(DEBUG_CONNECTION) System.err.println("returning from parseEventinfo ");
    return neweventList;
  }

  void parseInput(String s, int type) {
    DataItem tmp;

    try {
      switch(type) {
        case PERF_DATA_int:
          tmp = getNewDataItem();
          procmgr.getMetricParser(peekAtRank(s)).parseDATA(s,tmp);
          push(tmp);
          break;
        case PERF_BREACH_int:
          parseBREACH(s);
          break;
        case PERF_MHZ_int:
          parseMHZ(s);
          break;
        case PERF_OK_int:
          // ignore..
          break;
        default:
          System.err.println("Warning: Unsupported message type received (" 
             + type + ")!");
      }
    } catch(Exception e) {
      System.err.println("Warning: unable to process message! " + e.getMessage());
      e.printStackTrace();
    }
  }

  int peekAtRank(String s) {
    StringTokenizer st = new StringTokenizer(s, DataReader.STR_DEL + "@");

    return Integer.parseInt(st.nextToken()) - 1;
  }
 
  void parseMHZ(String s) {
    StringTokenizer st = new StringTokenizer(s, STR_DEL + RANK_DEL);

    if(DEBUG_CONNECTION) System.err.println(s);

    int rank = Integer.parseInt(st.nextToken()) - 1;
    double mhz = Double.valueOf(st.nextToken()).doubleValue();

    PerfProcessInfo pinf = procmgr.getProcInfo(rank);

    pinf.getHwInfo().mhz = mhz;
    procmgr.getMetricParser(rank).setProcInfo(pinf);
    //parser.setProcInfo(pinf);
  }

  void parseBREACH(String s) {
    StringTokenizer st = new StringTokenizer(s, STR_DEL + RANK_DEL);

    if(DEBUG_CONNECTION) System.err.println(s);

    int rank = Integer.parseInt(st.nextToken()) - 1;
    double min = Double.valueOf(st.nextToken()).doubleValue();
    double max = Double.valueOf(st.nextToken()).doubleValue();
    double actual = Double.valueOf(st.nextToken()).doubleValue();

    PerfBreachEvent evt = new PerfBreachEvent(this, rank, min, max, actual);

    for (Enumeration e = breachListeners.elements() ; e.hasMoreElements() ;) {
      ((BreachListener) e.nextElement()).alarmBreached(evt);
    }
  }

  public void addBreachListener(BreachListener l) {
    breachListeners.add(l);
  }

  public void removeBreachListener(BreachListener l) {
    breachListeners.remove(l);
  }

  private DataItem getNewDataItem() {
    synchronized(recycleBin) {
      if(recycleBin.isEmpty())
        return new DataItem();
      else
        return (DataItem) recycleBin.removeLast();
    }
  }

  private DataItem getNewDataItem(String a, String b, String c, String d,
    double e, int f, int g, String h) 
  {
    synchronized(recycleBin) {
      if(recycleBin.isEmpty())
        return new DataItem(a,b,c,d,e,f,g,h);
      else {
        DataItem temp = (DataItem) recycleBin.removeLast();
        temp.s1 = a;
        temp.s2 = b;
        temp.s3 = c;
        temp.s4 = d;
        temp.val = e;
        temp.color = f;
        temp.rank = g;
        temp.label = h;
        return temp;
      }
    }
  }

  private void push(Object o) {
    synchronized(itemBuffer) {
      itemBuffer.addLast(o);
    }
  }

  public DataItem getItem(DataItem d) {
    DataItem retval = null;

    synchronized(itemBuffer) {
      retval = (DataItem)itemBuffer.removeFirst();
    }

    synchronized(recycleBin) {
      recycleBin.addFirst(d);
    }

    return retval;
  }
}
