package edu.utk.cs.icl.perfometer.metrics;

import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.DataItem;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;

import java.util.StringTokenizer;

public class FlopPerCycleParser extends MetricParser {
  static final String [] metrics = { "PAPI_TOT_CYC", "PAPI_FP_INS" };
  static final String name = "Flop/cycle";
  double total_cycles = 0.0, total_fp_ins = 0.0;
  double min=Double.POSITIVE_INFINITY, max=Double.NEGATIVE_INFINITY;
 
  public MetricParser getNewInstance() {
    return new FlopPerCycleParser();
  }

  public void reset() {
    total_cycles = 0.0;
    total_fp_ins = 0.0;
    min=Double.POSITIVE_INFINITY;
    max=Double.NEGATIVE_INFINITY;
  }
    
  public String getMinMaxLabel() {
    return "Min/Max: ";
  }

  public String getMin() {
    return Double.toString(min);
  }

  public String getMax() {
    return Double.toString(max);
  }

  public String getTotalLabel() {
    return "Total FP Ins:";
  }
 
  public String getAggregateLabel() {
    return "Agg Flop/cycle";
  }

  public String getLabel() {
    return "Flop/cycle:";
  }
 
  public String getUnits() {
    return "";
  }

  public String getCommonName() {
    return name;
  }

  public String toString() {
    return name;
  }

  public String [] getMetricNames() {
    return metrics;
  }

  public void setProcInfo(PerfProcessInfo p) {
  }
  
  public void parseDATA(String s, DataItem dataItem) {
    StringTokenizer st = new StringTokenizer(s, DataReader.STR_DEL + "@");
    int num_results, num_data;
    double cycles, fpins, agg_mflop;

    dataItem.rank = Integer.parseInt(st.nextToken()) - 1;
 
    num_results = Integer.parseInt(st.nextToken());
  
    for(int i = 0; i < num_results; i++) {
      // ignore # data items for now
      num_data = Integer.parseInt(st.nextToken());

      dataItem.color = Integer.parseInt(st.nextToken());
      dataItem.label = st.nextToken();

      addLabel(dataItem.rank, dataItem.color, dataItem.label);

      dataItem.s1 = st.nextToken();
      dataItem.s2 = st.nextToken();

      cycles = Double.valueOf(st.nextToken()).doubleValue();
      fpins = Double.valueOf(st.nextToken()).doubleValue();

      total_cycles += cycles;
      total_fp_ins += fpins;

      agg_mflop = (int)((total_fp_ins / total_cycles)* 100) / 100.0;

      dataItem.s3 = Long.toString((long)total_fp_ins);
      dataItem.s4 = Double.toString(agg_mflop);

      dataItem.val = (int)((fpins / cycles) * 100) / 100.0;

      if(dataItem.val < min) min = dataItem.val;
      if(dataItem.val > max) max = dataItem.val;
    }
  }
}
