package edu.utk.cs.icl.perfometer.metrics;

import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.DataItem;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;

import java.util.StringTokenizer;

public class L2DataMissRatioParser extends MetricParser {
  static final String [] metrics = { "PAPI_L2_TCM", "PAPI_L2_TCA" };
  static final String name = "L2 Data Cache Miss Ratio";
  double total_miss=0.0, total_access=0.0;
  double min=Double.POSITIVE_INFINITY, max=Double.NEGATIVE_INFINITY;

  public MetricParser getNewInstance() {
    return new L2DataMissRatioParser();
  }

  public void reset() {
    total_miss=0.0;
    total_access=0.0;
    min=Double.POSITIVE_INFINITY;
    max=Double.NEGATIVE_INFINITY;
  }
    
  public String getMinMaxLabel() {
    return "Min/Max: ";
  }

  public String getMin() {
    return Double.toString(min);
  }

  public String getMax() {
    return Double.toString(max);
  }

  public String getTotalLabel() {
    return "Total L2 Misses:";
  }
 
  public String getAggregateLabel() {
    return "Agg Miss %:";
  }

  public String getLabel() {
    return "L2 Miss Ratio:";
  }

  public String getUnits() {
    return "%";
  }

  public String getCommonName() {
    return name;
  }

  public String toString() {
    return name;
  }

  public String [] getMetricNames() {
    return metrics;
  }

  public void setProcInfo(PerfProcessInfo p) {
  }
  
  public void parseDATA(String s, DataItem dataItem) {
    StringTokenizer st = new StringTokenizer(s, DataReader.STR_DEL + "@");
    int num_results, num_data;
    double n1, n2;

    dataItem.rank = Integer.parseInt(st.nextToken()) - 1;
 
    num_results = Integer.parseInt(st.nextToken());
  
    for(int i = 0; i < num_results; i++) {
      // ignore # data items for now
      num_data = Integer.parseInt(st.nextToken());

      dataItem.color = Integer.parseInt(st.nextToken());
      dataItem.label = st.nextToken();

      addLabel(dataItem.rank, dataItem.color, dataItem.label);

      dataItem.s1 = st.nextToken();
      dataItem.s2 = st.nextToken();

      n1 = Double.valueOf(st.nextToken()).doubleValue();
      n2 = Double.valueOf(st.nextToken()).doubleValue();

      total_miss += n1;
      total_access += n2;

      dataItem.s3 = Long.toString((long)total_miss);
      dataItem.s4 = Double.toString( ((int) ((total_miss/total_access)*10000))/100.0 );

      dataItem.val = ((int) ((n1/n2)*10000))/100.0;

      if(dataItem.val < min) min = dataItem.val;
      if(dataItem.val > max) max = dataItem.val;
    }
  }
}
