package edu.utk.cs.icl.perfometer.metrics;

import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.DataItem;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;

import java.util.StringTokenizer;

public class MemRefParser extends MetricParser {
  static final String [] metrics = { "PAPI_L2_TCM", "PAPI_L1_TCM" };
  static final String name = "Ratio of Memory References";
  double min=Double.POSITIVE_INFINITY, max=Double.NEGATIVE_INFINITY;

  public MetricParser getNewInstance() {
    return new MemRefParser();
  }

  public void reset() {
    min=Double.POSITIVE_INFINITY;
    max=Double.NEGATIVE_INFINITY;
  }
    
  public String getMinMaxLabel() {
    return "Min/Max: ";
  }

  public String getMin() {
    return Double.toString(min);
  }

  public String getMax() {
    return Double.toString(max);
  }

  public String getTotalLabel() {
    return "";
  }
 
  public String getAggregateLabel() {
    return "";
  }

  public String getLabel() {
    return "Mem ref ratio:";
  }

  public String getUnits() {
    return "%";
  }

  public String getCommonName() {
    return name;
  }

  public String toString() {
    return name;
  }

  public String [] getMetricNames() {
    return metrics;
  }

  public void setProcInfo(PerfProcessInfo p) {
  }
  
  public void parseDATA(String s, DataItem dataItem) {
    StringTokenizer st = new StringTokenizer(s, DataReader.STR_DEL + "@");
    int num_results, num_data;
    double n1, n2;

    dataItem.rank = Integer.parseInt(st.nextToken()) - 1;
 
    num_results = Integer.parseInt(st.nextToken());
  
    for(int i = 0; i < num_results; i++) {
      // ignore # data items for now
      num_data = Integer.parseInt(st.nextToken());

      dataItem.color = Integer.parseInt(st.nextToken());
      dataItem.label = st.nextToken();

      addLabel(dataItem.rank, dataItem.color, dataItem.label);

      dataItem.s1 = st.nextToken();
      dataItem.s2 = st.nextToken();
      dataItem.s3 = "0.0";
      dataItem.s4 = "0.0";

      n1 = Double.valueOf(st.nextToken()).doubleValue();
      n2 = Double.valueOf(st.nextToken()).doubleValue();

      dataItem.val = ((int) ((n1/n2)*10000))/100.0;

      if(dataItem.val < min) min = dataItem.val;
      if(dataItem.val > max) max = dataItem.val;
    }
  }
}
