package edu.utk.cs.icl.perfometer.micro_gui;

import edu.utk.cs.icl.perfometer.datatransfer.LineReader;
import edu.utk.cs.icl.perfometer.datatransfer.TraceFileReader;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;
import edu.utk.cs.icl.perfometer.datatransfer.PerfMetric;

import edu.utk.cs.icl.perfometer.metrics.*;

// import edu.utk.cs.icl.perfometer.event.BreachListener;
// import edu.utk.cs.icl.perfometer.event.PerfBreachEvent;

import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Panel;
import java.awt.FileDialog;
import java.awt.Toolkit;
import java.awt.Image;
import java.awt.Button;

import java.io.IOException;
import java.io.File;

import java.util.Enumeration;

public class MicroPerfometer extends Frame 
 implements java.awt.event.ActionListener
//  implements BreachListener
{
  public static final int CONNECTION_PRIORITY = Thread.MIN_PRIORITY;
  
  PerfPanel graph;
  Thread graphThread;
  Button traceFileButton, connectButton, exitButton, disconnectButton;
  Button pauseButton, metricButton;

  LineReader connection;

  Panel top_panel = new Panel();
  Panel bottom_panel = new Panel();

  PanelManager procmgr;

  boolean paused=false;

  private MetricParser [] highLevelMetrics = {
     new MflopParser(), new BranchMisPredictParser(),
     new L1DataMissRatioParser(), new L2DataMissRatioParser(),
     new FlopPerCycleParser()};

  public static void main (String [] args) {
    MicroPerfometer gui = new MicroPerfometer();
    gui.show();
  }

  public MicroPerfometer() {
    this.setSize(240,295);
    this.setTitle("PAPI Perfometer");

    traceFileButton = new Button("Trace File");
    connectButton = new Button("Connect");
    disconnectButton = new Button("Disconnect");
    exitButton = new Button("Exit");
    metricButton = new Button("Metric");
    pauseButton = new Button("Pause");

    top_panel.setLayout(new GridLayout(2,3));
    top_panel.add(connectButton);
    top_panel.add(disconnectButton);
    top_panel.add(traceFileButton);
    top_panel.add(metricButton);
    top_panel.add(pauseButton);
    top_panel.add(exitButton);

    bottom_panel.setLayout(new GridLayout(1,0));

    this.setLayout(new BorderLayout(5,5));
    this.add("North", top_panel);
    this.add("Center", bottom_panel);

    traceFileButton.addActionListener(this);
    connectButton.addActionListener(this);
    disconnectButton.addActionListener(this);
    exitButton.addActionListener(this);
    metricButton.addActionListener(this);
    pauseButton.addActionListener(this);
  }

  InputBox connInput = new InputBox(this, "Connect", false, highLevelMetrics);

  public void actionPerformed(java.awt.event.ActionEvent e) {
    Object src = e.getSource();

    if(src == exitButton) {
      System.exit(1);
    }
    else if(src == disconnectButton) {
      destroyConnection();
    }
    else if(src == connectButton) {
      connInput.show();

      if(!connInput.lastCancelled) {
        if(connection != null)
          destroyConnection();

        establishConnection(connInput.hostname, connInput.hostname, 
            connInput.portNum, false, false);
      }
    }
    else if(src == traceFileButton) {
      openTraceFile();
    }
    else if(src == metricButton) {
      changeMetric();
    }
    else if(src == pauseButton) {
      pauseGraph();
    }
  }

  void pauseGraph() {
    paused = !paused;

    if(paused)
      pauseButton.setLabel("Resume");
    else
      pauseButton.setLabel("Pause");

    pauseAllGraphs(paused);
  }

  void changeMetric() {
    MetricSelectionDialog md = null;
    PerfPanel p = null;
    PerfProcessInfo procinfo = null;

    pauseAllGraphs(true);

    try {
      // get the first element, since we only graph one thing
      p = (PerfPanel) procmgr.elements().nextElement();
      procinfo = p.getProcInfo();

      md = new MetricSelectionDialog(this, highLevelMetrics, p.getMetrics());

      md.show();

      MetricParser metric = md.getSelectedMetric();

      procmgr.changeAllMetrics(metric);

      if(metric != null) {
        try {
          clearAllGraphs();
          connection.sendChangeMetric(p.getRank(), metric);

          if(procinfo != null) {
            MetricParser mp = procmgr.getMetricParser(p.getRank());
            mp.setProcInfo(procinfo);
          }

          relabelAllGraphs();
        }catch(UnsupportedMetricException e) {
           ErrorMessage d = new ErrorMessage(this, "Error",
               "Unsupported Metric: " + e.getMessage());
           d.show();
        }
      }
    }
    catch(Exception e) {
      ErrorMessage d = new ErrorMessage(this, "Error",
          "Error changing metric: " + e.getMessage());
      d.show();
    }

    pauseAllGraphs(false);
  }

  public void clearAllGraphs() {
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      perfpanel.clearPoints();
    }
  }

  public void relabelAllGraphs() {
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      perfpanel.initDisplay();
    }
  }
  public void pauseAllGraphs(boolean s) {
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      perfpanel.setPaused(s);
    }
  }

  void destroyConnection() {
    if(graph != null) graph.stopGraphing();
    if(graphThread != null) graphThread.interrupt();

    bottom_panel.removeAll();
    graph = null;
    graphThread = null;

    try { connection.disconnect(); } catch (Exception e) {} 

    connection = null;
  }

  void openTraceFile()
  {
    MetricParser metric = new MflopParser();
    FileDialog fd = new FileDialog(this, "Open Trace File", FileDialog.LOAD);

    paused = false;
    pauseButton.setLabel("Pause");

    metricButton.setEnabled(false);

    destroyConnection();

    fd.show();

    if(fd.getFile() == null)
      return;

    File tracefile = null;

    try{
      tracefile = new File(fd.getDirectory() + fd.getFile());
    } catch (Exception e) {
      ErrorMessage d = new ErrorMessage(this, "Error",
          "Error opening trace file: " + e.getMessage());
      d.show();
      return;
    }

    try {
      procmgr = new PanelManager();
      connection = new TraceFileReader(tracefile, procmgr);

      ((TraceFileReader)connection).establishConnection();
      sendAllMetrics(metric);
      connection.setPriority(CONNECTION_PRIORITY);

      procmgr.changeAllMetrics(metric);
      if(procmgr.numElements() > 1)
        throw new IOException("The CE client can only handle one graph.");

      graph = (PerfPanel)procmgr.elements().nextElement();
      bottom_panel.add(graph);
      bottom_panel.validate();
      graph.setSize(bottom_panel.getSize());
      graphThread = new Thread(graph, "Performance Graph");
      graphThread.start();

      connection.start();
    }catch(Exception exc) {
      if(connection != null) {
        connection.disconnect();
        connection = null;
      }
      ErrorMessage d = new ErrorMessage(this, "Error",
          "Error connecting: " + exc.getMessage());
      d.show();
    }
  }

  void establishConnection(String connectTo, String host, int portNum,
    boolean isLocal, boolean noDialog)
  {
    MetricParser metric = new MflopParser();

    destroyConnection();

    paused = false;
    pauseButton.setLabel("Pause");

    metricButton.setEnabled(true);

    try {
      procmgr = new PanelManager();
      connection = new LineReader(procmgr, metric);

      metric = connInput.getSelectedMetric();

//       connection.addBreachListener(this);

      connection.establishConnection(connectTo, portNum);
      sendAllMetrics(metric);
      connection.setPriority(CONNECTION_PRIORITY);

      procmgr.changeAllMetrics(metric);

      if(procmgr.numElements() > 1)
        throw new IOException("The CE client can only handle one graph.");

//      graph = new PerfPanel(connection, connectTo, 20, 0, 
//        connection.getProgramInfo()[0].name); 
      graph = (PerfPanel)procmgr.elements().nextElement();
      bottom_panel.add(graph);
      bottom_panel.validate();
      graph.setSize(bottom_panel.getSize());
      graphThread = new Thread(graph, "Performance Graph");
      graphThread.start();

      connection.start();
    }catch(Exception exc) {
      if(connection != null) {
        connection.disconnect();
        connection = null;
      }
      ErrorMessage d = new ErrorMessage(this, "Error",
          "Error connecting: " + exc.getMessage());
      d.show();
    }
  }

  public void sendAllMetrics(MetricParser p)
   throws UnsupportedMetricException
  {
    PerfProcessInfo procinfo;
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      procinfo = perfpanel.getProcInfo();

      connection.sendChangeMetricUnsynchronized(perfpanel.getRank(), p);

      if(procinfo != null) {
        MetricParser mp = procmgr.getMetricParser(perfpanel.getRank());
        mp.setProcInfo(procinfo);
      }
    }
  }
}
