package edu.utk.cs.icl.perfometer.micro_gui;

import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.ProcessManager;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;
import edu.utk.cs.icl.perfometer.event.BreachListener;
import edu.utk.cs.icl.perfometer.metrics.*;

import java.util.Hashtable;
import java.util.Enumeration;

import java.io.IOException;

public class PanelManager implements ProcessManager {
  public static final int DEFAULT_HISTORY = 40;

  private Hashtable panelTable;
  private DataReader dataReader;
  private int numPanels = 0;

  public PanelManager(int init_size) {
    panelTable = new Hashtable(init_size);
  }

  public void setMetricParser(int rank, MetricParser m) {
    PerfPanel p = (PerfPanel) panelTable.get(Integer.toString(rank));

    p.setMetricParser(m);
  }

  public PanelManager() {
    panelTable = new Hashtable(DEFAULT_TABLE_SIZE);
  }

  public void setDataReader(DataReader d) {
    dataReader = d;
  }

  public Enumeration elements() {
    return panelTable.elements();
  }

  public int numElements() {
    return numPanels;
  }

  public void changeAllMetrics(MetricParser m) {
    for (Enumeration e = elements() ; e.hasMoreElements() ;) {
      ((PerfPanel) e.nextElement()).setMetricParser(m.getNewInstance());
    }
  }

  public MetricParser getMetricParser(int rank) {
    PerfPanel p = (PerfPanel) panelTable.get(Integer.toString(rank));

    if(p == null) return null;

    return p.getMetricParser();
  }

  public PerfProcessInfo getProcInfo(int rank)
  {
    PerfPanel p = (PerfPanel) panelTable.get(Integer.toString(rank));

    if(p == null) return null;

    return p.getProcInfo();
  }

  public void add(int rank, PerfProcessInfo pinfo, MetricParser m)
  {
    PerfPanel p;

    try {
      p = new PerfPanel(dataReader, pinfo, DEFAULT_HISTORY, rank);

      panelTable.put(Integer.toString(rank), p);

      numPanels++;
    } catch(Exception e) {
      System.err.println("Unable to create new PerfPanel: "
        + e.getMessage());
    }
  }

  public void remove(int rank)
  {
    try {
      panelTable.remove(Integer.toString(rank));
      numPanels--;
    } catch(Exception e) {
      System.err.println("Unable to create new PerfPanel: "
        + e.getMessage());
    }
  }

  public Object get(int rank)
  {
    PerfPanel p = (PerfPanel) panelTable.get(Integer.toString(rank));

    if(p == null) return null;

    return p;
  }
}
