package edu.utk.cs.icl.perfometer.micro_gui;

// 
// Project: PAPI - Perfometer Visualization Tool
// Author:  Nathan P. Garner
//          garner@cs.utk.edu
// Date:    Oct. 1999
//  

import java.awt.Panel;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Color;
import java.awt.Polygon;
import java.awt.Image;

class PerfHist extends Panel implements Runnable {
  int PAPI_LINEAR = 0;
  int PAPI_LOG = 1;
  int PAPILogLinear = PAPI_LINEAR;

  public static final int BARGRAPH = 0;
  public static final int XYSCATTER = 1;
  public static final int LINEGRAPH = 2;
  public static final int LINEGRAPHWITHPOINTS = 3;
  public static final int FILLEDLINE = 4;

  int graphType = FILLEDLINE;

  String graphLabel = "";
  double curMax = 0.0;
  double [][]values;
  int historyLength = 0;
  int lru;
  int readElements;

  Image buffer;         // variables for double buffering
  Graphics gbuffer;

  Thread graphThread;

  Font graphFont = new Font("SansSerif", Font.PLAIN, 10);

  public PerfHist(int h, double [][]v) {
    this.historyLength = h;
    this.values = v;
    readElements = 0;
    lru = 0;

    this.setVisible(true);
  }

  public Dimension getMinimumSize() {
    return new Dimension(historyLength + 20, 30);
  }

  public Font getGraphFont() {
    return graphFont;
  }

  public void setGraphFont(Font f) {
    graphFont = f;
  }

  public synchronized void start() {
    if (graphThread == null) {
      graphThread = new Thread(this);
      graphThread.start();
    }
  }

  public void clearPoints() {
    lru = readElements = 0;
  }

  public void setHistoryLength (int l) {
    historyLength = l;
  }

  public void setValues(double [][] v) {
    values = v;
  }

  public double [][] getValues() {
    return values;
  }

  public void incLRU() {
    lru++;
  }

  public void setLRU(int i) {
    lru =i;
  }

  public int getPixPerCol(int drawWidth) {
    double r = (double)drawWidth/(double)historyLength;

    return (r < 1.0) ? 1 : (int) r;
  }

  public double getCurMax() {
    return curMax;
  }

  public synchronized void stop() {
    notify();
  }

  public void run() {
  }

  public void setReadElements(int n) {
    readElements = n;
  }

  public int getReadElements() {
    return readElements;
  }

  public void incReadElements() {
    readElements++;
  }

  public void setGraphType(int n) {
    graphType = n;
  }

  public int getGraphType() {
    return graphType;
  }

  public int getHistoryLength() {
    return historyLength;
  }

  public double calcMax() {
    double max = 0.0;

    for (int loop = 0; loop < readElements; loop++)
      if ( max < values[loop][0]) max = values[loop][0];

    if (curMax == 0.0)
      curMax = max;

    if (max > curMax) {
      while (max > curMax) {
        if (PAPILogLinear == PAPI_LINEAR)
          curMax = curMax * 2.0;
        else
          curMax = curMax * 10.0;
      }
    } else if (max < (curMax * 0.5)) {
      while (max < (curMax * 0.5))
        curMax = curMax * 0.5;
    }
    max = curMax;

    // -------------------------------------------------
    // Short term fix for Percentage displays
    if (graphLabel.equalsIgnoreCase("Hit Rate")) {
      max = 100;
      curMax = 100;
    }
 
    return max;
  }

  public void setGraphLabel(String t) {
    graphLabel = t;
  }

  public String getGraphLabel() {
    return graphLabel;
  }
    
  void setLogLinearScale(int n) {
    PAPILogLinear = n;
  }

  public void paintForeground ( Graphics gbuffer ) {

    if(gbuffer == null) return;

    Container c = getParent();
    if(c != null)
      this.setSize(c.getSize());

    gbuffer.setFont(getGraphFont());

    int readElements = getReadElements();
    int historyLength = getHistoryLength();

    int x=0, y=0;
    int height = this.getSize().height;

    int pixelsPerColumn = getPixPerCol(this.getSize().width -
            gbuffer.getFontMetrics().stringWidth(Integer.toString((int)(calcMax() + 0.5))) - 2);
    double heightScale = (height - gbuffer.getFontMetrics().getHeight() + 5) / calcMax();

    int prevY = 0;
    int curIdx = 0;
    int barHeight = 0;
    int xp = 0;
    int gtype = getGraphType();

    if (readElements > 0) {
      // Adapt algorithm for the first few elements

      if(pixelsPerColumn <= 1)
        pixelsPerColumn = 1;

      x = x + (pixelsPerColumn * (historyLength - readElements));

      values = getValues();

      prevY = height - (int)(values[lru][0] * heightScale);

      // Paint the bars
      for (int loop = lru; loop < lru + readElements; loop++) {
        curIdx = loop % historyLength;

        barHeight = (int)(values[curIdx][0] * heightScale);
        y = height - barHeight;
                
        xp = x + pixelsPerColumn;

        switch (gtype) {
          case PerfHist.BARGRAPH:
            // Draw black border around each bar to distinguish them from each other

            gbuffer.setColor(new Color((int)values[curIdx][1])); 
            gbuffer.fillRect(x, y, pixelsPerColumn, barHeight);
            gbuffer.setColor(Color.black); 
            gbuffer.drawRect(x, y, pixelsPerColumn, barHeight);
            break;

          case PerfHist.XYSCATTER :
            gbuffer.setColor(Color.black); 
            gbuffer.fillRect(x, y, 3, 3);
            break;

          case PerfHist.LINEGRAPH : 
            gbuffer.setColor(Color.black); 
            gbuffer.drawLine(x, prevY, xp, y);
            break;

          case PerfHist.LINEGRAPHWITHPOINTS : 
            gbuffer.setColor(Color.black); 
            gbuffer.drawLine(x, prevY, xp, y);
            gbuffer.fillRect(xp-1, y-1, 3, 3);
            break;
                        
          case PerfHist.FILLEDLINE:
            if((int)values[curIdx][1] == 0)
              gbuffer.setColor(Color.red); 
            else
              gbuffer.setColor(new Color((int)values[curIdx][1])); 

            if(pixelsPerColumn == 1) {
              gbuffer.drawLine(x, height, x, y);
            }
            else {
              Polygon p = new Polygon();
              p.addPoint(x, height);
              p.addPoint(x, prevY);
              p.addPoint(xp, y);
              p.addPoint(xp, height);
              gbuffer.fillPolygon(p);
 
              gbuffer.setColor(Color.black);
              gbuffer.drawPolygon(p);
              gbuffer.fillRect(xp-1, y-1, 3, 3);                                                                            
            }
            break;
        }

        prevY = y;

        x = xp;
      }
    }
  }

  public final static int incr = 10;

  public void update(Graphics g) {
    repaint(g);
  }

  public void repaint(Graphics g) {
    Dimension d = this.getSize();
    Image i;

    if((d.width ==0) || (d.height==0)) return;

    i = this.createImage(d.width, d.height);

    try {
       
      paintBackground(i.getGraphics());
      paintForeground(i.getGraphics());

      //foregroundPane.getGraphics().drawImage(i,0,0,null);
      g.drawImage(i,0,0,null);

    } catch (NullPointerException e) {
       // no graphics area - ignore
    }
  }

  public void paintBackground(Graphics g) {

    if(g == null) return;

    Container c = getParent();
    if(c != null)
      this.setSize(c.getSize());

    Dimension d = this.getSize();
    g.clearRect(0, 0, d.width, d.height);

    g.setFont(getGraphFont());

    int height = getSize().height;

    int historyLength = getHistoryLength();
    double max = calcMax();

    // Turn the current max into String for display on graph
    String maxValueStr = Integer.toString((int)(max+0.5));

    // Draw the current scale values
    FontMetrics metrics = g.getFontMetrics();
    int strHeight = metrics.getHeight();

    int pixelsPerColumn = getPixPerCol(getSize().width - metrics.stringWidth( maxValueStr ) - 2);

    int insideTop = strHeight + 5;
    int insideRight = pixelsPerColumn*historyLength;

    g.setColor(Color.black);

    // Put the maximum value text label and line on the screen
    g.drawLine(0, insideTop, insideRight, insideTop);
    g.drawLine(insideRight, 0, insideRight, height);
    g.drawString(maxValueStr, insideRight + 2, insideTop);

    // Put 1/2 * max value text label on the screen
    g.drawString(Integer.toString(((int)(max+0.5)) >> 1), insideRight + 2, 
                 ((height + insideTop) >> 1) + (strHeight >> 1));

    // Put the zero point text label on the screen
    g.drawString( "0" , insideRight + 2, height - 2);

    // Finished Drawing scale values
    // Now draw the label for the graph
    g.drawString( getGraphLabel(), (insideRight >> 1) - (metrics.stringWidth( getGraphLabel() ) >> 1) , 
         strHeight + 3);

    g.setColor(Color.gray);

    for (int i=0; i < (insideRight + incr); i += incr)
      g.drawLine(i, height, i, insideTop);

    for (int i=insideTop; i < (height + incr); i += incr)
      g.drawLine(0, i, insideRight, i);
  }
}
