package edu.utk.cs.icl.perfometer.micro_gui;

import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.DataItem;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;
import edu.utk.cs.icl.perfometer.datatransfer.PerfHwInfo;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProgramInfo;
import edu.utk.cs.icl.perfometer.datatransfer.PerfMetric;

import edu.utk.cs.icl.perfometer.metrics.*;

import com.sun.java.util.collections.*;

import java.awt.Panel;
import java.awt.Container;
import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.Label;

import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.IOException;

public class PerfPanel extends Panel implements Runnable {

  private int historyLength = 50; // Default # of bars to display
  private double[][] values; // values to be displayed and the color codes
  private int internalDelay = 0;  // time to sleep before drawing the next bar
  private boolean paused = false;  // is the pause button is pressed?
  private boolean stop = false;
  private boolean ds_stopped = false;

  private MetricParser metric = null;

  private int PAPIPollingIntervalms;
  private int PAPIPollingIntervalSec;
    
  private final int LINEAR = 0;
  private final int LOG = 1;
  private int PAPILogLinear = LINEAR;

  private int curPos = 0;
  private PerfHist perfHist = null;
    
  private String graphLabel = null;

  private String h1 = null, h2 = null, l1 = null, l2 = null, l3 = null;
  private String u1 = null, u2 = null, u3 = null;

  private DataReader datareader;

  private int procNum;

  private PrintWriter pout;
  private BufferedReader rdr;

  private PerfProcessInfo procinfo;

  public PerfPanel(DataReader r, PerfProcessInfo p, int h, int n) throws IOException
  {
    super();

    Container c = getParent();
    if(c != null) setSize(c.getSize());

    datareader = r;
    procinfo = p;

    graphLabel = procinfo.getExeInfo().name;

    procNum = n;
 
    historyLength = h;
    values = new double [historyLength][2];
        
    setupBarDisplay();

    perfHist.setGraphLabel(graphLabel);
    perfHist.setVisible(true);
    perfHist.setGraphFont(new Font("SansSerif", Font.PLAIN, 10));

    initDisplay();

    this.setLayout(new BorderLayout(5,5));
    Label l = new Label(h2);
    l.setFont(new Font("SansSerif", Font.PLAIN, 10));
    this.add("North", l);
    this.add("Center", perfHist);

    this.setVisible(true);
  }

  public void clearPoints() {
    curPos = 0; 
    perfHist.clearPoints();
  }

  public int getRank() {
    return procNum;
  }

  public void setMetricParser(MetricParser m) {
    metric = m;
    initDisplay();
  }

  public MetricParser getMetricParser() {
    return metric;
  }

  public PerfProcessInfo getProcInfo() {
    return procinfo;
  }

  public PerfMetric [] getMetrics() {
    return procinfo.getEventList();
  }

  public PerfHist getPerfHist() {
    return perfHist;
  }

  void setupBarDisplay() {
    perfHist = new PerfHist(historyLength, values);
    perfHist.setVisible(true);
  }
    
  public void run() { 
    DataItem d = new DataItem();

    for(;;) {

      if(stop) return;

      while(paused || ds_stopped) {
        if(stop) return;
        try { Thread.sleep(300); }
        catch(InterruptedException e) { if(stop) return; }
      }

      if(internalDelay > 0) {
        try { Thread.sleep(internalDelay); }
        catch(InterruptedException e) { if(stop) return; }
      }
      else
        Thread.yield();

        try {
//          d = datareader.getItem(procNum, d);
          d = datareader.getItem(d);
        }catch(NoSuchElementException e){
          try { Thread.sleep(500); }
          catch(InterruptedException e2) { if(stop) return; }
          continue;
        }

      values[curPos][0] = d.val;
      values[curPos][1] = d.color;

      if (perfHist.getReadElements() < historyLength) {
        perfHist.incReadElements();
        perfHist.lru = 0;
      } else {
        perfHist.lru++;
      }
      curPos++;
 
      // Roll back around to beginning of history
      if (perfHist.lru == historyLength) perfHist.lru = 0;
      if (curPos == historyLength) curPos = 0;          

//      cf.setValues(d.s1, d.s2, d.s3, Double.toString(d.val));

      perfHist.repaint();
    }
  }
    
  int getInternalDelay() {
    return internalDelay;
  }

  void setInternalDelay(int n) {
    internalDelay = n;
//    cf.setInternalDelay(n);
  }

  void setGraphType(int n) {
    perfHist.setGraphType(n);
  }

  void setLogLinearScale(int n) {
    PAPILogLinear = n;
    perfHist.setLogLinearScale(n);
  }

  int getLogLinearScale() {
    return PAPILogLinear;
  }
    
  int getHistoryLength() {
    return historyLength;
  }

  public void setStopped(boolean x) {
    try {
      if(x)
        datareader.sendDatastreamStop(procNum);
      else
        datareader.sendDatastreamStart(procNum);
    } catch (IOException e) {
      System.err.println("Cannot send stop message: " +
         e.getMessage());
    }

    ds_stopped = x;
  }

  public void setPaused(boolean x) {
    try {
      if(x)
        datareader.sendPause(procNum);
      else
        datareader.sendStart(procNum);
    } catch (IOException e) {
      System.err.println("Cannot send stop message: " +
         e.getMessage());
    }

    paused = x;
  }

  public void stopGraphing() {
    stop = true;
  }

  public boolean isStopped() {
    return ds_stopped;
  }

  public boolean isPaused() {
    return paused;
  }

  void setHistoryLength(int n) {
    double[][] newvalues;
    int upperBound;

//System.err.println();
//System.err.println("setting history length to " + n);
//System.err.println("readElements = " +perfHist.getReadElements());
//System.err.println("perfHist.lru = " +perfHist.lru);
//System.err.println("historyLength = " +historyLength);
//System.err.println("curPos = " +curPos );

    paused = true;

    newvalues = new double [n][2];

    if (perfHist.getReadElements() < historyLength)
      upperBound = perfHist.getReadElements();
    else
      upperBound = perfHist.lru + historyLength;

    int j = 0;
    for(int i = perfHist.lru; (i < upperBound) && (i < n); i++, j++) {
      newvalues[j][0] = values[i % historyLength][0];
      newvalues[j][1] = values[i % historyLength][1];
    }

    historyLength = n;
    values = newvalues;

    perfHist.setReadElements(j);
    curPos = j-1;

    perfHist.setHistoryLength(historyLength);
    perfHist.setValues(values);
    perfHist.setVisible(true);
    perfHist.setGraphLabel(graphLabel);
    	
//System.err.println("going to unpause now");
    paused = false;
  }

  // Put the information into the labels onscreen
  void initDisplay() {

    PerfHwInfo hwinfo = procinfo.getHwInfo();
    PerfProgramInfo exeinfo = procinfo.getExeInfo();
    
    if(exeinfo.name.length() == 0)
      h1 = "Process: <unknown>"; 
    else
      h1 = "Process: " + exeinfo.name;

    h2 = hwinfo.machinename + ", a " + hwinfo.totalcpus 
     + " CPU " + hwinfo.model_string + " at " + hwinfo.mhz + " Mhz.";

    if(metric != null)
      perfHist.setGraphLabel(hwinfo.machinename + " -- " + metric.getLabel());
    else
      perfHist.setGraphLabel(hwinfo.machinename);

    l1 = "Real time";
    u1 = "s.";
    l2 = "Process time:";
    u2 = "s.";
    l3 = "Total:";
    u3 = "FP Ins.";

//    cf.setLabels(h1, h2, l1, l2, l3, graphLabel);
//    cf.setUnits(u1, u2, u3, null);
  }
}
