package edu.utk.cs.icl.perfometer.swing_gui;

import edu.utk.cs.icl.perfometer.datatransfer.AppInput;
import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.LineReader;

import java.util.*;
import java.awt.Toolkit.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

public class ConnectionEstablishmentDialog extends JDialog 
  implements AppInput, PerfConnListener
{
  JTextArea hostnameTxt = new JTextArea(10, 40);
  JLabel JLabel1 = new JLabel();
  JPanel JPanel2 = new JPanel();
  JPanel mainPanel = new JPanel();
  JPanel connPanel = new JPanel();
  JButton okBtn = new JButton();
  JButton cancelBtn = new JButton();

  String remoteMachine;
  int remotePort;

  String [] hostArray;

  ConnectionHandler connThread;
  DataReader newConnection;
  JButton cancelConnectionButton;
  JLabel label;
  JLabel machineLabel;

  boolean okPressed = false;

  public static final int CONN_INTERRUPTED   = -2;
  public static final int CONN_ABNORMAL      = -1;
  public static final int CONN_ESTABLISHING  = 0;
  public static final int CONN_NORMAL        = 1;

  int status = CONN_ESTABLISHING;

  String err = null;

  ConnectionEstablishmentDialog(JFrame parent, DataReader conn,
      String machine, int port)
  {
    super(parent, "Connecting", true);
    
    Container contentPane = getContentPane();

    setModal(true);
    //setSize(400,280);
    setSize(350,120);

    getContentPane().setLayout(new BorderLayout(20,20));
    //contentPane.setLayout(new CardLayout());

    /*** set up the initial connection dialog ***/
    cancelConnectionButton = new JButton("Cancel");
    label = new JLabel("Establishing connection to: ");
    machineLabel = new JLabel(machine + ":" + port);
    machineLabel.setForeground(Color.blue);

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

    JPanel middlePanel = new JPanel();
    middlePanel.setLayout(new FlowLayout(FlowLayout.CENTER));

    JPanel bottomPanel = new JPanel();
    bottomPanel.setLayout(new FlowLayout(FlowLayout.CENTER));

    topPanel.add(label);
    middlePanel.add(machineLabel);
    bottomPanel.add(cancelConnectionButton);

    connPanel.setSize(350,120);
    connPanel.setLayout(new BoxLayout(connPanel, BoxLayout.Y_AXIS));
    connPanel.add(topPanel);
    connPanel.add(middlePanel);
    connPanel.add(bottomPanel);

    contentPane.add(BorderLayout.CENTER, connPanel);

    contentPane.setSize(connPanel.getSize());
    SymAction lSymAction = new SymAction();
    okBtn.addActionListener(lSymAction);
    cancelBtn.addActionListener(lSymAction);
    cancelConnectionButton.addActionListener(lSymAction);

    newConnection = conn;
    remoteMachine = machine;
    remotePort = port;
  }

  public void startConnecting()
  {
    connThread =
      new ConnectionHandler(this, newConnection, remoteMachine, remotePort);

    connThread.start();

    this.show();
  }

  class SymAction implements java.awt.event.ActionListener
  {
    public void actionPerformed(java.awt.event.ActionEvent event)
    {
      Object object = event.getSource();
      if (object == okBtn)
        okBtn_actionPerformed(event);
      else if (object == cancelBtn)
        cancelBtn_actionPerformed(event);
      else if (object == cancelConnectionButton)
        cancelConnectionButton_actionPerformed(event);
    }
  }

  public String perror() {
    return err;
  }

  public int getStatus() {
    return status;
  }

  public void connectionNormal() {
    status = CONN_NORMAL;
    this.dispose();
  }

  public void connectionInterrupted() {
    status = CONN_INTERRUPTED;
    this.dispose();
  }

  public void connectionFailed(String reason) {
    // if we already got a user interruption, then
    // dont reset status to abnormal
    if(status != CONN_INTERRUPTED)
      status = CONN_ABNORMAL;

    err = reason;
    this.dispose();
  }

  public void waitForHosts() {
    Container cp = getContentPane();

    cp.remove(connPanel);
    this.setSize(400,280);
    cp.setSize(400,280);
    mainPanel.setSize(400,280);

    /*** now set up the host input dialog ***/

    JLabel1.setHorizontalAlignment(SwingConstants.LEFT);
    JLabel1.setText("Enter Host Names/Ports, one per line:");

    okBtn.setNextFocusableComponent(cancelBtn);
    okBtn.setText("OK");
    okBtn.setActionCommand("OK");

    cancelBtn.setText("Cancel");
    cancelBtn.setActionCommand("Cancel");

    JPanel2.setLayout(new FlowLayout(FlowLayout.RIGHT,5,5));
    JPanel2.add(okBtn);
    JPanel2.add(cancelBtn);

    mainPanel.setSize(400,280);
    mainPanel.setLayout(new BorderLayout(5,5));
    mainPanel.setBorder(new EmptyBorder(10,10,10,10));

    mainPanel.add(BorderLayout.NORTH,JLabel1);
    mainPanel.add(BorderLayout.CENTER,hostnameTxt);
    mainPanel.add(BorderLayout.SOUTH,JPanel2);

    cp.add(BorderLayout.CENTER, mainPanel);

    // cheesy...
    while(!okPressed) {
      try { connThread.sleep(300); } catch (InterruptedException e) {} 
    }
  }

  public String [] getHosts() {
    return hostArray;
  }

  void okBtn_actionPerformed(java.awt.event.ActionEvent event)
  {
    StringTokenizer st = new StringTokenizer(hostnameTxt.getText(), "\n");

    hostArray = new String[st.countTokens()];

    int i = 0;

    while(st.hasMoreTokens())
      hostArray[i++] = st.nextToken(); 

    okPressed = true;
    return;
  }

  // not sure if the same button can exist in different cards or not..
  // but we just use two different cancel buttons to do the same thing.

  void cancelConnectionButton_actionPerformed(java.awt.event.ActionEvent event)
  {
    connThread.interrupt();
    connectionInterrupted();
    return;
  }

  void cancelBtn_actionPerformed(java.awt.event.ActionEvent event)
  {
    connThread.interrupt();
    connectionInterrupted();
    return;
  }
}

interface PerfConnListener {
  public void connectionNormal();
  public void connectionInterrupted();
  public void connectionFailed(String reason);
}

class ConnectionHandler extends Thread {
  DataReader conn;
  String host;
  int port;
  PerfConnListener listener;

  public ConnectionHandler(PerfConnListener l, DataReader c,
    String h, int p)
  {
    listener = l;
    conn = c;
    host = h;
    port = p;
  }

  public void run() {

    try {
      conn.establishConnection(host, port);
      listener.connectionNormal();
    } catch(Throwable t) {
      listener.connectionFailed(t.getMessage());
    }
  }
}
