package edu.utk.cs.icl.perfometer.swing_gui;

import java.awt.event.*;
import java.util.Enumeration;

import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.DataItem;

public class GraphUpdateThread extends Thread implements ActionListener {
  public static final int VERY_SLOW = 2000;
  public static final int SLOW = 1000;
  public static final int MEDIUM = 600;
  public static final int FAST = 400;
  public static final int VERY_FAST = 70;

  private PanelManager procmgr = null;
  private DataReader datareader = null;
  private boolean stop = false;
  private int drawDelay = MEDIUM;

  javax.swing.Timer timer;

  public GraphUpdateThread(DataReader d, PanelManager pm) {
    datareader = d;
    procmgr = pm;

    timer = new javax.swing.Timer(drawDelay, this);
    timer.setInitialDelay(0);
    timer.setCoalesce(true);
  }

  public void setDrawingDelay(int ms) {
    drawDelay = ms;
    timer.setDelay(drawDelay);
  }

  public void stopGraphing() {
    stop = true;
  }

  public void run() {
    DataItem d = new DataItem();
    PerfPanel perfPanel = null;

    // sleep for a bit to give the event dispatch thread some time
    // to finish painting all the gui components before we start
    // sending a bunch of repaint requests for the graphs.
    try { Thread.sleep(700); } catch(InterruptedException e) {}
    timer.start();

    for(;;) {

      if(stop) return;

      try {
        d = datareader.getItem(d);
        perfPanel = (PerfPanel) procmgr.get(d.rank);
        perfPanel.addPoint(d);
      }catch(Exception e){
        try { Thread.sleep(100); }
        catch(InterruptedException e2) { if(stop) return; }
        continue;
      }
    }
  }

  public void actionPerformed(ActionEvent evt) {
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      perfpanel.repaint();
      // calling initdisplay mainly to handle the case
      // where MHZ might change.
      perfpanel.initDisplay();
    }
  }
}
