package edu.utk.cs.icl.perfometer.swing_gui;

import edu.utk.cs.icl.perfometer.datatransfer.PerfMetric;
import edu.utk.cs.icl.perfometer.metrics.*;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;

public class MetricSelectionDialog extends JDialog
{
  MetricParser[] eventList;
  MetricParser selectedMetric = null;
    
  PerfMetric [] lowLevelEvents;

  JPanel buttonPanel = new JPanel();
  JPanel instructionPanel = new JPanel();
  JPanel listPanel = new JPanel();

  JTextArea instrPane = new JTextArea();

  JList highLevelEventList;
  JList lowLevelEventList;

  CardLayout listCards;

  JButton okBtn = new JButton();
  JButton CancelBtn = new JButton();
  JButton advancedBtn = new JButton();

  TitledBorder titledBorder1 = new TitledBorder(new LineBorder(Color.gray, 2), "Metric Selection");
  TitledBorder titledBorder5 = new TitledBorder(new LineBorder(Color.gray, 2), "Metric to be displayed");

  String instructions = "The PAPI metrics that can be monitored on your application are shown below under available metrics.  Please select the metric to be shown on the graph, then select the metrics that will be displayed as text.";

  static final String ADV_LABEL = "Show Advanced Metrics";
  static final String HIGH_LABEL = "Show Basic Metrics";

  public MetricSelectionDialog(MetricParser[] el, PerfMetric [] low) {
    eventList = el;
    lowLevelEvents = low;
        
    setTitle("Metric Selection");
    setSize(626, 300);
    setModal(true);
    getContentPane().setLayout(new BorderLayout(5,5));

    highLevelEventList = new JList(new DefaultListModel());
    lowLevelEventList = new JList(new DefaultListModel());

    highLevelEventList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    lowLevelEventList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    JScrollPane highLevelScrollPane = new JScrollPane(highLevelEventList);
    JScrollPane lowLevelScrollPane = new JScrollPane(lowLevelEventList);

    okBtn.setText("OK");
    okBtn.setActionCommand("OK");

    CancelBtn.setText("Cancel");
    CancelBtn.setActionCommand("Cancel");

    advancedBtn.setText(ADV_LABEL);
    advancedBtn.setActionCommand("advanced");

    buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT,5,5));
    buttonPanel.add(advancedBtn);
    buttonPanel.add(okBtn);
    buttonPanel.add(CancelBtn);

    listPanel.setBorder(titledBorder5);
    listCards = new CardLayout();
    listPanel.setLayout(listCards);
    listPanel.add(highLevelScrollPane, "high");
    listPanel.add(lowLevelScrollPane, "low");
    listCards.show(listPanel, "high");

    instrPane.setText(instructions);
    instrPane.setOpaque(false);
    instrPane.setEditable(false);
    instrPane.setLineWrap(true);
    instrPane.setWrapStyleWord(true);

    instructionPanel.setBorder(titledBorder1);
    instructionPanel.setLayout(new BoxLayout(instructionPanel, BoxLayout.Y_AXIS));
    instructionPanel.add(instrPane);

    getContentPane().add(BorderLayout.NORTH, instructionPanel);
    getContentPane().add(BorderLayout.CENTER, listPanel);
    getContentPane().add(BorderLayout.SOUTH,buttonPanel);

    setupLists();
            
    SymAction lSymAction = new SymAction();
    okBtn.addActionListener(lSymAction);
    CancelBtn.addActionListener(lSymAction);
    advancedBtn.addActionListener(lSymAction);
  }

  public MetricParser getSelectedMetric() {
    return selectedMetric;
  }

  void setupLists() {
    MetricParser e;
    PerfMetric l;
    DefaultListModel listModel;

    listModel = (DefaultListModel) highLevelEventList.getModel();

    for(int i = 0; i < eventList.length; i++) {
      e = eventList[i];
      if(e != null)
        listModel.addElement(e);
    }

    listModel = (DefaultListModel) lowLevelEventList.getModel();

    for(int i = 0; i < lowLevelEvents.length; i++) {
      l = lowLevelEvents[i];
      if(l != null)
        listModel.addElement(l);
    }
  }

  class SymAction implements java.awt.event.ActionListener
  {
    public void actionPerformed(java.awt.event.ActionEvent event)
    {
      Object object = event.getSource();

      if (object == CancelBtn)
        CancelBtn_actionPerformed(event);
      else if (object == okBtn)
        okBtn_actionPerformed(event);
      else if (object == advancedBtn) {
        if(advancedBtn.getText().equals(ADV_LABEL))
          advancedBtn.setText(HIGH_LABEL);
        else
          advancedBtn.setText(ADV_LABEL);

        listCards.next(listPanel);
      }
    }
  }

  void CancelBtn_actionPerformed(java.awt.event.ActionEvent event)
  {
    selectedMetric = null;
    this.setVisible(false);
    return;
  }

  void okBtn_actionPerformed(java.awt.event.ActionEvent event)
  {
    if(advancedBtn.getText().equals(ADV_LABEL))
      selectedMetric = (MetricParser) highLevelEventList.getSelectedValue();
    else {
      PerfMetric m = (PerfMetric) lowLevelEventList.getSelectedValue();
      GenericParser g = new GenericParser();

      g.setMetricNames(new String [] {m.getName()});
      g.setTotalLabel("Total: ");
      g.setAggregateLabel("Aggregate: ");
      g.setLabel(m.getName() + ": ");
      g.setUnits("");
      g.setCommonName(m.getName());

      selectedMetric = g;
    }

    this.setVisible(false);
    return;
  }

}

