package edu.utk.cs.icl.perfometer.swing_gui;

import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.ProcessManager;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;
import edu.utk.cs.icl.perfometer.event.BreachListener;
import edu.utk.cs.icl.perfometer.metrics.MetricParser;

import java.util.Enumeration;
import java.util.Hashtable;

import java.io.IOException;

public class PanelManager implements ProcessManager {
  public static final int DEFAULT_HISTORY = 100;

  private Hashtable panelTable;
  private DataReader dataReader;
  private int numPanels = 0;

  private double graphScale = 0.0;

  public PanelManager(int init_size) {
    panelTable = new Hashtable(init_size);
  }

  public PanelManager() {
    panelTable = new Hashtable(DEFAULT_TABLE_SIZE);
  }

  public void setDataReader(DataReader d) {
    dataReader = d;
  }

  public Enumeration elements() {
    return panelTable.elements();
  }

  public int numElements() {
    return numPanels;
  }

  public double getGraphScale() {
    double val, max = 0.0;

    for (Enumeration e = elements() ; e.hasMoreElements() ;) {
      PerfPanel tmp = (PerfPanel) e.nextElement();
      
      val = tmp.getMaxValue();

      if(val > max)
        max = val;
    }

    if (graphScale == 0.0) {
      graphScale = max;
      return graphScale;
    }

    if (max > graphScale)
      graphScale = max * 2.0;
    else if (max < (graphScale * 0.5))
      graphScale = max * 0.5;

    return graphScale;
  }

  public void add(int rank, PerfProcessInfo pinfo, MetricParser m)
  {
    PerfPanel p;

    try {
      p = new PerfPanel(dataReader, pinfo, DEFAULT_HISTORY, rank, m);

      panelTable.put(Integer.toString(rank), p);

      numPanels++;
    } catch(Exception e) {
      Perfometer.errLog("Unable to create new PerfPanel: "
        + e.getMessage());
      e.printStackTrace();
    }
  }

  public void remove(int rank)
  {
    try {
      panelTable.remove(Integer.toString(rank));
      numPanels--;
    } catch(Exception e) {
      Perfometer.errLog("Unable to remove PerfPanel: "
        + e.getMessage());
      e.printStackTrace();
    }
  }

  public Object get(int rank)
  {
    PerfPanel p = (PerfPanel) panelTable.get(Integer.toString(rank));

    if(p == null) return null;

    return p;
  }

  public PerfProcessInfo getProcInfo(int rank)
  {
    PerfPanel p = (PerfPanel) panelTable.get(Integer.toString(rank));

    if(p == null) return null;

    return p.getProcInfo();
  }

  public void changeAllMetrics(MetricParser m) {
    for (Enumeration e = elements() ; e.hasMoreElements() ;) {
      ((PerfPanel) e.nextElement()).setMetricParser(m.getNewInstance());
    }
  }

  public void setMetricParser(int rank, MetricParser m) {
    PerfPanel p = (PerfPanel) panelTable.get(Integer.toString(rank));

    p.setMetricParser(m);
  }

  public MetricParser getMetricParser(int rank) {
    PerfPanel p = (PerfPanel) panelTable.get(Integer.toString(rank));

    if(p == null) return null;

    return p.getMetricParser();
  }
}
