package edu.utk.cs.icl.perfometer.swing_gui;

import edu.utk.cs.icl.perfometer.metrics.MetricParser;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Container;
import java.awt.Image;

import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JComponent;

public class PerfHist extends JLayeredPane {
  public static final int GRAPH_LINEAR = 0;
  public static final int GRAPH_LOG    = 1;

  public static final int BARGRAPH = 0;
  public static final int XYSCATTER = 1;
  public static final int LINEGRAPH = 2;
  public static final int LINEGRAPHWITHPOINTS = 3;
  public static final int FILLEDLINE = 4;

  private int graphScale = GRAPH_LINEAR;
  private int graphType = FILLEDLINE;
  private int gridWidth = 10;

  private String graphLabel = "";
  private double curMax = 0.0;
  private GraphValues values;
  protected int historyLength;

  private GraphBackgroundPane backgroundPane;
  private GraphForegroundPane foregroundPane;

  private Font graphFont = new Font("SansSerif", Font.PLAIN, 10);

  private String [] overlay_text;

  public static Color greyFilter = new Color(40,40,40,100);
  private Color filterColor = null;

  private int uniqueID;

  private boolean avgEnabled = true;

  private PanelManager panelMgr;

  public PerfHist(int h) {
    historyLength = h;
    values = new GraphValues(h);

    panelMgr = Perfometer.getInstance().getPanelManager();

    backgroundPane = new GraphBackgroundPane(this);
    foregroundPane = new GraphForegroundPane(this);
      
    this.add(backgroundPane, new Integer(0));
    this.add(foregroundPane, new Integer(1));

    backgroundPane.setVisible(true);
    foregroundPane.setVisible(true);

    this.setVisible(true);
  }


  public void setUniqueID(int id) {
    uniqueID = id;
  }
   
  public int getUniqueID() {
    return uniqueID;
  }

  public MetricParser getMetricParser() {
    return panelMgr.getMetricParser(uniqueID);
  }

  public void addPoint(double val, double color, double a) {
    synchronized(values) {
      values.values[values.curPos][0] = val;
      values.values[values.curPos][1] = color;
      values.agg[values.curPos] = a;

      if (values.readElements < historyLength) {
        values.readElements++;
        values.lru = 0;
      } else {
        values.lru++;
      }

      values.curPos++;

      // Roll back around to beginning of history
      if (values.lru == historyLength) values.lru = 0;
      if (values.curPos == historyLength) values.curPos = 0;
    }
  }

  public void setGridWidth(int w) {
    gridWidth = w;
  }

  public int getGridWidth() {
    return gridWidth;
  }

  public void clearPoints() {
    synchronized(values) {
      values.readElements = 0;
      values.curPos = 0;
      values.lru = 0;
    }
  }

  public void setFilterEnabled(boolean x) {
    if(x)
      filterColor = greyFilter;
    else
      filterColor = null;
  }

  public Color getFilterColor() {
    return filterColor;
  }

  public void getValues(GraphValues g) {
    synchronized(values) {
       System.arraycopy(values.values, 0, g.values, 0, values.values.length);
       System.arraycopy(values.agg, 0, g.agg, 0, values.agg.length);
       g.lru = values.lru;
       g.readElements = values.readElements;
       g.curPos = values.curPos;
    }
  }

  public void setOverlayText(String [] s) {
    overlay_text = s;
    foregroundPane.setOverlayText(s);
  }

  public void clearOverlayText() {
    overlay_text = null;
    foregroundPane.clearOverlayText();
  }

  public Dimension getMinimumSize() {
    return new Dimension(historyLength + 20, 30);
  }

  public Font getGraphFont() {
    return graphFont;
  }

  public void setGraphFont(Font f) {
    graphFont = f;
  }

  public void updateGraph() {
    Dimension d = this.getSize();
    Image i;
    Graphics g;

    if((d.width ==0) || (d.height==0)) return;

    i = backgroundPane.createImage(d.width, d.height);

    try {
      backgroundPane.paintComponent(i.getGraphics());
      foregroundPane.paintComponent(i.getGraphics());
       
      foregroundPane.getGraphics().drawImage(i,0,0,null);

    } catch (NullPointerException e) {
       // no graphics area - ignore
    }
  }

/*
  void setHistoryLength(int n) {
    double[][] newvalues;
    int upperBound;

    paused = true;

    newvalues = new double [n][2];

    if (getReadElements() < historyLength)
      upperBound = perfHist.getReadElements();
    else
      upperBound = perfHist.lru + historyLength;

    int j = 0;
    for(int i = perfHist.lru; (i < upperBound) && (i < n); i++, j++) {
      newvalues[j][0] = values[i % historyLength][0];
      newvalues[j][1] = values[i % historyLength][1];
    }

    historyLength = n;
    values = newvalues;

    perfHist.setReadElements(j);
    curPos = j-1;

    setHistoryLength(historyLength);
    setValues(values);
    setVisible(true);
    setAlignmentX(Component.CENTER_ALIGNMENT);
    setGraphLabel(graphLabel);

    paused = false;
  }

*/

  public void setHistoryLength (int l) {
    historyLength = l;
  }

  public int getPixPerCol(int drawWidth) {
    double r = (double)drawWidth/(double)historyLength;

    return (r < 1.0) ? 1 : (int) r;
  }

  public void setGraphType(int n) {
    graphType = n;
  }

  public int getGraphType() {
    return graphType;
  }

  public int getHistoryLength() {
    return historyLength;
  }

  public double getMaxValue() {
    double max = 0.0;

    // perhaps keep track somewhere else

    for (int loop = 0; loop < values.readElements; loop++) {
      if ( max < values.values[loop][0]) max = values.values[loop][0];
      if ( max < values.agg[loop]) max = values.agg[loop];
    }

    return max;
  }

  public double calcMax() {
    Perfometer perf = Perfometer.getInstance();

    if(perf.scalesLocked())
      return panelMgr.getGraphScale();

    double max = getMaxValue();
  
    if (curMax == 0.0)
      curMax = max;
  
    if (max > curMax) {
      if (graphScale == GRAPH_LINEAR)
        curMax = max * 2.0;
      else
        curMax = max * 10.0;
    } else if (max < (curMax * 0.5)) {
      curMax = max * 0.5;
    }
  
    max = curMax;
  
    return max;
  }

  public void setGraphLabel(String t) {
    graphLabel = t;
  }

  public String getGraphLabel() {
    return graphLabel;
  }
    
  public int getLogLinearScale() {
    return graphScale;
  }

  public void setLogLinearScale(int n) {
    graphScale = n;
  }

  public boolean getAvgEnabled() {
    return avgEnabled;
  }

  public void setAvgEnabled(boolean avg) {
    avgEnabled = avg;
  }

}

class GraphValues {
  public double [][]values;
  public double []agg;
  public int curPos, lru, readElements;

  public GraphValues(int size) {
    values = new double[size][2];
    agg = new double[size];
    curPos = 0;
    lru = 0;
    readElements = 0;
  }
}

class GraphForegroundPane extends JPanel {
  GraphValues values;
  String [] overlay_text;

  PerfHist parent;

  int borderPad = 5;
  int fontPad = 1;

  public GraphForegroundPane(PerfHist p) {
    super(true);
    parent = p;
    this.setOpaque(false);
    this.setSize(300,300);
    this.setVisible(true);

    values = new GraphValues(parent.getHistoryLength());
  }

  public void clearOverlayText() {
    overlay_text = null;
  }

  public void setOverlayText(String [] s) {
    overlay_text = s;
  }

  public void paintComponent ( Graphics gbuffer ) {

    if(gbuffer == null) return;

    Container c = getParent();
    if(c != null)
      this.setSize(c.getSize());

    gbuffer.setFont(parent.getGraphFont());

    int historyLength = parent.getHistoryLength();

    int x = 0, y = 0, agg_y = 0;
    int height = this.getSize().height;

    int pixelsPerColumn = parent.getPixPerCol(this.getSize().width -
            gbuffer.getFontMetrics().stringWidth(
               Integer.toString((int)(parent.calcMax() + 0.5))) - 2);

    double heightScale = (height - (gbuffer.getFontMetrics().getHeight() + 5)) / parent.calcMax();

    int prevY = 0;
    int curIdx = 0;
    int barHeight = 0;
    int xp = 0;
    int gtype = parent.getGraphType();

    parent.getValues(values);

    if (values.readElements > 0) {
      // Adapt algorithm for the first few elements

      if(pixelsPerColumn <= 1)
        pixelsPerColumn = 1;

      x = x + (pixelsPerColumn * (historyLength - values.readElements));

      prevY = height - (int)(values.values[values.lru][0] * heightScale);

      // Paint the bars
      for (int loop = values.lru; loop < values.lru + values.readElements; loop++) {
        curIdx = loop % historyLength;

        barHeight = (int)(values.values[curIdx][0] * heightScale);
        y = height - barHeight;
                
        xp = x + pixelsPerColumn;

        switch (gtype) {
          case PerfHist.BARGRAPH:
            // Draw black border around each bar to distinguish them from each other

            if((int)values.values[curIdx][1] == 0)
              gbuffer.setColor(Color.red); 
            else
              gbuffer.setColor(Color.decode(Integer.toString((int)values.values[curIdx][1]))); 
            gbuffer.fillRect(x, y, pixelsPerColumn, barHeight);
            gbuffer.setColor(Color.black); 
            gbuffer.drawRect(x, y, pixelsPerColumn, barHeight);
            break;

          case PerfHist.XYSCATTER :
            gbuffer.setColor(Color.black); 
            gbuffer.fillRect(x, y, 3, 3);
            break;

          case PerfHist.LINEGRAPH : 
            gbuffer.setColor(Color.black); 
            gbuffer.drawLine(x, prevY, xp, y);
            break;

          case PerfHist.LINEGRAPHWITHPOINTS : 
            gbuffer.setColor(Color.black); 
            gbuffer.drawLine(x, prevY, xp, y);
            gbuffer.fillRect(xp-1, y-1, 3, 3);
            break;
                        
          case PerfHist.FILLEDLINE:
            if((int)values.values[curIdx][1] == 0)
              gbuffer.setColor(Color.red); 
            else
              gbuffer.setColor(Color.decode(Integer.toString((int)values.values[curIdx][1]))); 

            if(pixelsPerColumn == 1) {
              gbuffer.drawLine(x, height, x, y);
            }
            else {
              int [] xpoints = {x,x,xp,xp};
              int [] ypoints = {height,prevY,y,height};
              gbuffer.fillPolygon(xpoints, ypoints, 4);
              gbuffer.setColor(Color.black);
              gbuffer.drawPolygon(xpoints, ypoints, 4);
              gbuffer.fillRect(xp-1, y-1, 3, 3);                                                                            
            }
            break;
        }

        if(parent.getAvgEnabled()) {
          int diam = Math.max(2 , pixelsPerColumn - 2);

          agg_y = height - (int)(values.agg[curIdx] * heightScale) - (diam/2);

          gbuffer.setColor(Color.yellow); 
          gbuffer.fillOval(x, agg_y, diam, diam);
          if(diam > 1) {
            gbuffer.setColor(Color.black); 
            gbuffer.drawOval(x, agg_y, diam, diam);
          }
        }

        prevY = y;

        x = xp;
      }
    }

    Color filter = parent.getFilterColor();

    if(filter != null)
    {
      int insideRight = pixelsPerColumn*historyLength;

      gbuffer.setColor(filter);
      gbuffer.fillRect(0,gbuffer.getFontMetrics().getHeight()+borderPad,insideRight,getSize().height);
    }

    if(overlay_text != null)
      drawOverlayText(gbuffer, overlay_text);
  }

  private void drawOverlayText(Graphics gbuffer, String [] cstr) {
    if((cstr == null) || (cstr.length < 1))
      return;

    FontMetrics fm = gbuffer.getFontMetrics();
 
    String maxValueStr = Integer.toString((int)(parent.calcMax()+0.5));

    int historyLength = parent.getHistoryLength();
    int pixelsPerColumn = parent.getPixPerCol(getSize().width - fm.stringWidth( maxValueStr ) - 2);
    int insideRight = pixelsPerColumn*historyLength;
    int fontHeight = fm.getHeight() + fontPad;

    int x_origin = insideRight / 2;
    int y_origin = this.getHeight() / 2;

    int maxWidth = 0;

    for(int i=0;i < cstr.length; i++) {
      int thisStrWidth = fm.stringWidth(cstr[i]);

      if(thisStrWidth > maxWidth)
        maxWidth = thisStrWidth;
    }

    int strheight = fontHeight * cstr.length;

    gbuffer.setColor(Color.yellow);
    gbuffer.fillRect(
      (x_origin) - (maxWidth/2) - borderPad,
      (y_origin) - (strheight/2) - borderPad,
      maxWidth + (borderPad * 2), strheight + (borderPad * 2));

    gbuffer.setColor(Color.red);

    gbuffer.drawRect(
      (x_origin) - (maxWidth/2) - borderPad,
      (y_origin) - (strheight/2) - borderPad,
      maxWidth + (borderPad * 2), strheight + (borderPad * 2));

    int x_off = x_origin - (maxWidth/2);

    y_origin = (y_origin + (fontHeight/3)) - (cstr.length - 1) * (fontHeight / 2);

    for(int i=0;i < cstr.length; i++) {
      gbuffer.drawString(cstr[i], x_off, y_origin);
      y_origin += fontHeight;
    }
  }
}

class GraphBackgroundPane extends JComponent {
  PerfHist parent;

  public GraphBackgroundPane(PerfHist p) {
    super();
    parent = p;
    this.setSize(300,300);
  }

  public void paintComponent(Graphics g) {

    if(g == null) return;

    String unit = "";

    Container c = getParent();
    if(c != null)
      this.setSize(c.getSize());

    g.setFont(parent.getGraphFont());

    int height = getSize().height;

    int historyLength = parent.getHistoryLength();
    double max = parent.calcMax();

    Perfometer perf = Perfometer.getInstance();
    try {
      //unit = perf.getCurrentMetric().getUnits();
      unit = parent.getMetricParser().getUnits();
    } catch(Exception e) {
      unit = "";
    }

    // Turn the current max into String for display on graph
    String maxValueStr = Integer.toString((int)(max+0.5)) + unit;

    // Draw the current scale values
    FontMetrics metrics = g.getFontMetrics();
    int strHeight = metrics.getHeight();

    int pixelsPerColumn = parent.getPixPerCol(getSize().width - metrics.stringWidth( maxValueStr ) - 2);

    int insideTop = strHeight + 5;
    int insideRight = pixelsPerColumn*historyLength;

    g.setColor(Color.black);

    // Put the maximum value text label and line on the screen
    g.drawLine(0, insideTop, insideRight, insideTop);
    g.drawLine(insideRight, 0, insideRight, height);
    g.drawString(maxValueStr, insideRight + 2, insideTop);

    // Put 1/2 * max value text label on the screen
    g.drawString(Integer.toString(((int)(max+0.5)) >> 1) + unit, insideRight + 2, 
                 ((height + insideTop) >> 1) + (strHeight >> 1));

    // Put the zero point text label on the screen
    g.drawString( "0" +unit, insideRight + 2, height - 2);

    // Finished Drawing scale values
    // Now draw the label for the graph
    g.drawString( parent.getGraphLabel(), (insideRight >> 1) - (metrics.stringWidth( parent.getGraphLabel() ) >> 1) , 
         strHeight + 3);

    g.setColor(Color.gray);

    int incr = parent.getGridWidth();

    for (int i=0; i < (insideRight + incr); i += incr)
      g.drawLine(i, height, i, insideTop);

    for (int i=insideTop; i < (height + incr); i += incr)
      g.drawLine(0, i, insideRight, i);
  }
}
