package edu.utk.cs.icl.perfometer.swing_gui;

import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.DataItem;
import edu.utk.cs.icl.perfometer.datatransfer.AlarmSettings;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;
import edu.utk.cs.icl.perfometer.datatransfer.PerfHwInfo;
import edu.utk.cs.icl.perfometer.datatransfer.PerfMetric;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProgramInfo;

import edu.utk.cs.icl.perfometer.metrics.MetricParser;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import javax.swing.border.*;
import java.awt.event.*;

import java.net.*;
import java.io.*;
import java.text.*;
import javax.swing.*;
import java.lang.*;

public class PerfPanel extends JPanel {
    // Variables required for communication with the server
  private PerfHist miniGraph = null;

  private PerfProcessInfo procinfo = null;

  private MetricParser metric = null;

  private AlarmSettings alarmSettings = null;

  private int historyLength = 50; // Default # of bars to display
  private int pollingInterval = 100;

  private boolean paused = false;  // is the pause button pressed?
  private boolean ds_stopped = false;

  private ControlsFrame cf = null;
  private PerfHist perfHist = null;
    
  private String graphLabel = null;

  private String h1 = null, h2 = null, l1 = null, l2 = null, l3 = null;
  private String u1 = null, u2 = null, u3 = null;

  private DataReader datareader;

  private int procNum;

  public PerfPanel(DataReader r, PerfProcessInfo p, int h, int n, MetricParser m)
     throws IOException
  {
    super();

    datareader = r;
    procinfo = p;
    procNum = n;
    historyLength = h;
    metric = m;

    graphLabel = procinfo.getExeInfo().name;

    cf = new ControlsFrame(this);
    cf.setAlignmentX(Component.LEFT_ALIGNMENT);
    cf.setMaximumSize(cf.getPreferredSize());
    cf.setAlarmEnabled(datareader.isServer());
      
    setupBarDisplay();

    initDisplay();

    this.setLayout(new BorderLayout());
    this.add(cf, BorderLayout.NORTH);
    this.add(perfHist, BorderLayout.CENTER);

    cf.setAlignmentX(Component.CENTER_ALIGNMENT);

    cf.setVisible(true);

    this.setVisible(true);
  }

  public void setMetricParser(MetricParser m) {
    metric = m;
  }

  public MetricParser getMetricParser() {
    return metric;
  }

  public int getRank() {
    return procNum;
  }

  public String getGraphLabel() {
    return graphLabel;
  }

  public PerfHist getMiniGraph() {
    return miniGraph;
  }

  public void clearAlarm() {
    alarmSettings = null;

    cf.setPauseEnabled(true);
    cf.setStopEnabled(true);

    perfHist.setFilterEnabled(false);
    miniGraph.setFilterEnabled(false);

    perfHist.setOverlayText(null);
    miniGraph.setOverlayText(null);

    perfHist.repaint();
    miniGraph.repaint();
  }

  public void setAlarmSettings(AlarmSettings set) {
    alarmSettings = set;

    try {
      datareader.sendPopupParameters(procNum, set.min,
        set.max, set.thresh, 3);

      cf.setPauseEnabled(false);
      cf.setStopEnabled(false);
    } catch (IOException e) {
      Perfometer.errLog("Error sending popup parameters: "
        + e.getMessage());
    }

    String [] breach_text = {"BREACH WAIT",
       "Min = " + set.min, "Max = " + set.max,
       "Thresh = " + set.thresh};

    perfHist.setOverlayText(breach_text);
    miniGraph.setOverlayText(breach_text);
    perfHist.setFilterEnabled(true);
    miniGraph.setFilterEnabled(true);

    perfHist.clearPoints();
    miniGraph.clearPoints();

    perfHist.repaint();
    miniGraph.repaint();
  }

  public void clearPoints() {
    perfHist.clearPoints();
    miniGraph.clearPoints();
  }

  public AlarmSettings getAlarmSettings() {
    return alarmSettings;
  }

  public PerfHist getPerfHist() {
    return perfHist;
  }

  void setupBarDisplay() {
    perfHist = new PerfHist(historyLength);
    perfHist.setVisible(true);
    perfHist.setAlignmentX(Component.CENTER_ALIGNMENT);
    perfHist.setGraphLabel(graphLabel);
    perfHist.setGraphFont(new Font("SansSerif", Font.PLAIN, 14));

    miniGraph = new PerfHist(historyLength);
    miniGraph.setVisible(true);
    miniGraph.setAlignmentX(Component.CENTER_ALIGNMENT);
    miniGraph.setGraphLabel(graphLabel);
    miniGraph.setUniqueID(procNum);
    miniGraph.setGraphFont(new Font("SansSerif", Font.PLAIN, 10));
  }
    
  public double getMaxValue() {
    if(perfHist == null) return 100.0;

    return perfHist.getMaxValue();
  }

  public void addPoint(DataItem d) {
    if( paused || ds_stopped || (alarmSettings != null))
      return;

    Perfometer perf = Perfometer.getInstance();
    //MetricParser metric = perf.getCurrentMetric();

    perfHist.addPoint(d.val,d.color,Double.valueOf(d.s4).doubleValue());
    miniGraph.addPoint(d.val,d.color,Double.valueOf(d.s4).doubleValue());

    cf.setValues(d.s1, d.s2, d.s3, Double.toString(d.val), d.s4, 
        metric.getMin() + "/" + metric.getMax());

    //perfHist.repaint();
    //miniGraph.repaint();
  }

  void setGraphType(int n) {
    perfHist.setGraphType(n);
    miniGraph.setGraphType(n);

    perfHist.repaint();
    miniGraph.repaint();
  }

  void setLogLinearScale(int n) {
    perfHist.setLogLinearScale(n);
    miniGraph.setLogLinearScale(n);
  }

  int getLogLinearScale() {
    return perfHist.getLogLinearScale();
  }
    
  public boolean getAvgEnabled() {
    return perfHist.getAvgEnabled();
  }

  public void setAvgEnabled(boolean avg) {
    if(perfHist != null)
      perfHist.setAvgEnabled(avg);
    if(miniGraph != null)
      miniGraph.setAvgEnabled(avg);
  }

  int getHistoryLength() {
    return historyLength;
  }

  public int getPollingIntervalms() {
    return pollingInterval;
  }

  public int getPollingIntervalSec() {
    return pollingInterval / 1000;
  }

  public void setPollingIntervalms(int n) {
    if(n < 0)
      return;

    pollingInterval = n;
    datareader.sendPollingInterval(procNum,n);
  }

  public void setPollingIntervalSec(int idx, int n) {
    setPollingIntervalms(n*1000);
  }

  public void repaint() {
    if(perfHist != null)
      perfHist.repaint();
    if(miniGraph != null)
      miniGraph.repaint();
  }
 
  public void setStopped(boolean x) {
    try {
      if(x)
        datareader.sendDatastreamStop(procNum);
      else
        datareader.sendDatastreamStart(procNum);
    } catch (IOException e) {
      Perfometer.errLog("Cannot send stop message: " +
         e.getMessage());
    }

    ds_stopped = x;
  }

  public void setPaused(boolean x) {
    try {
      if(x)
        datareader.sendPause(procNum);
      else
        datareader.sendStart(procNum);
    } catch (IOException e) {
      Perfometer.errLog("Cannot send stop message: " +
         e.getMessage());
    }

    paused = x;
  }

  public boolean isStopped() {
    return ds_stopped;
  }

  public boolean isPaused() {
    return paused;
  }

  public PerfProcessInfo getProcInfo() {
    return procinfo;
  }

  public PerfMetric[] getMetrics() {
    return procinfo.getEventList();
  }

  // Put the information into the labels onscreen
  void initDisplay() {
    Perfometer perf = Perfometer.getInstance();
    PerfHwInfo hwinfo = procinfo.getHwInfo();
    PerfProgramInfo exeinfo = procinfo.getExeInfo();

    //metric = perf.getCurrentMetric(); 
    
    if(exeinfo.name.length() == 0)
      h1 = "Process: <unknown>"; 
    else
      h1 = "Process: " + exeinfo.name;

    h2 = "Machine: " + hwinfo.machinename + ", a " + hwinfo.totalcpus 
     + " CPU " + hwinfo.model_string + " at " + hwinfo.mhz + " Mhz.";

    l1 = "Real time";
    u1 = "s.";
    l2 = "Process time:";
    u2 = "s.";

    cf.setLabels(h1, h2, l1, l2, metric.getTotalLabel(), metric.getLabel(),
         metric.getAggregateLabel(), metric.getMinMaxLabel());
    cf.setUnits(u1, u2, "", metric.getUnits(), "", metric.getUnits());
  }
}
