package edu.utk.cs.icl.perfometer.swing_gui;

import edu.utk.cs.icl.perfometer.datatransfer.AlarmSettings;
import edu.utk.cs.icl.perfometer.datatransfer.LineReader;
import edu.utk.cs.icl.perfometer.datatransfer.DataReader;
import edu.utk.cs.icl.perfometer.datatransfer.TraceFileReader;
import edu.utk.cs.icl.perfometer.datatransfer.PerfProcessInfo;
import edu.utk.cs.icl.perfometer.datatransfer.DataItem;
import edu.utk.cs.icl.perfometer.datatransfer.PerfMetric;

import edu.utk.cs.icl.perfometer.event.BreachListener;
import edu.utk.cs.icl.perfometer.event.PerfBreachEvent;

import edu.utk.cs.icl.perfometer.metrics.*;

import java.net.URL;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.io.*;

public class Perfometer extends JFrame implements BreachListener
{
  public static final String PAPI_BARGRAPH = "BARGRAPH";
  public static final String PAPI_XYSCATTER = "XYSCATTER";
  public static final String PAPI_LINEGRAPH = "LINEGRAPH";
  public static final String PAPI_LINEGRAPHWITHPOINTS = "LINEGRAPHWITHPOINTS";
  public static final String PAPI_FILLEDLINE = "FILLEDLINE";

  private InputBox connectionInputBox;
  private MetricParser defaultMetric = new MflopParser();

  public static boolean perfdebug = false;

  private MiniGraphMouseListener mgMouseListener;

  private GraphUpdateThread graphingThread = null;

  final int BARINCREMENT = 10;
  static int histBars = 100;
  private LineReader connection = null;

  private String prevHost = "localhost";

  private JFileChooser fc = new JFileChooser();

  public static final int CONNECTION_PRIORITY = Thread.MIN_PRIORITY;
  public static final int GRAPHING_PRIORITY = CONNECTION_PRIORITY+1;

  // Used by addNotify
  boolean frameSizeAdjusted = false;
  boolean startedFromCmdLine = false;

  JSplitPane PerfDesktop = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true);

  JTabbedPane tabbedConnPane = new JTabbedPane(SwingConstants.TOP);

  JPanel miniGraphs = new JPanel();

  ImageIcon disconnectIcon;
  ImageIcon connectIcon;
  ImageIcon connectLocalIcon;
  ImageIcon exitIcon;
  ImageIcon metricIcon;
  ImageIcon legendIcon;
  ImageIcon largeICLIcon;

  JToolBar JToolBar1 = new JToolBar();
  JButton defaultConnectBtn = new JButton();
  JButton connectLocalButton = new JButton();
  JButton connectButton = new JButton();
  JButton disconnectButton = new JButton();
  JButton metricButton = new JButton();
  JButton legendButton = new JButton();
  JToolBar.Separator JToolBarSeparator1 = new JToolBar.Separator();
  JToolBar.Separator JToolBarSeparator2 = new JToolBar.Separator();
  JButton exitButton = new JButton();
  JMenuBar MainMenu = new JMenuBar();
  JMenu ConnectionMenu = new JMenu();
  JMenuItem ConnectMenu = new JMenuItem();
  JMenuItem ConnectLocalMenu = new JMenuItem();
  JMenuItem DisconnectMenu = new JMenuItem();
  JMenuItem ExitMenu = new JMenuItem();
  JMenuItem TraceFileMenu = new JMenuItem();
  JMenu OptionsMenu = new JMenu();
  JMenuItem pollingIntervalMenu = new JMenuItem();
  JRadioButtonMenuItem lockScalesMenu = new JRadioButtonMenuItem();
  JMenuItem logLinearScaleMenu = new JMenuItem();
  JMenuItem alarmsMenu = new JMenuItem();

  JMenu HelpMenu = new JMenu();
  JMenuItem aboutMenuItem = new JMenuItem();
  JMenuItem helpMenuItem = new JMenuItem();

  private PanelManager procmgr;

  private MetricParser [] highLevelMetrics = { 
     new MflopParser(), new BranchMisPredictParser(),
     new L1DataMissRatioParser(), new L2DataMissRatioParser(),
     new FlopPerCycleParser()};

  protected Perfometer()
  {
    setTitle("Perfometer");
    setJMenuBar(MainMenu);
    getContentPane().setLayout(new BorderLayout(0,0));
    setSize(970,600);
    setVisible(false);

    try {
      ClassLoader l = Perfometer.class.getClassLoader();

      disconnectIcon = new ImageIcon(l.getResource("images/disconnect.gif"));
      connectIcon = new ImageIcon(l.getResource("images/connect.gif"));
      connectLocalIcon = new ImageIcon(l.getResource("images/connectlocal.gif"));
      exitIcon = new ImageIcon(l.getResource("images/exit.gif"));
      metricIcon = new ImageIcon(l.getResource("images/metric.gif"));
      legendIcon = new ImageIcon(l.getResource("images/legend.gif"));
      largeICLIcon = new ImageIcon(l.getResource("images/large_icl.gif"));
    }
    catch (Exception iconloading_exc) {}

    miniGraphs.setLayout(new BoxLayout(miniGraphs, BoxLayout.X_AXIS));

    JToolBar1.setAlignmentY(0.222222F);

    getContentPane().add(BorderLayout.NORTH, JToolBar1);

    defaultConnectBtn.setDefaultCapable(false);
    defaultConnectBtn.setToolTipText("Quick Connect - Localhost on default port number.");

    connectLocalButton.setDefaultCapable(false);
    connectLocalButton.setToolTipText("Connect to a local process on a given port");
    connectLocalButton.setText("Connect Local  ");
    connectLocalButton.setActionCommand("Connect Localhost");

    connectButton.setDefaultCapable(false);
    connectButton.setToolTipText("Connect to a process on the Inter/Intranet");
    connectButton.setText("Connect to Net  ");
    connectButton.setActionCommand("Connection");

    disconnectButton.setDefaultCapable(false);
    disconnectButton.setToolTipText("Disconnect from the running process");
    disconnectButton.setText("Disconnect  ");
    disconnectButton.setActionCommand("Disconnect");

    metricButton.setDefaultCapable(false);
    metricButton.setText("Select Metric");
    metricButton.setActionCommand("metricsButton");

    metricButton.setEnabled(true);

    legendButton.setDefaultCapable(false);
    legendButton.setText("Show Legend");
    legendButton.setActionCommand("legendButton");

    legendButton.setEnabled(true);

    exitButton.setDefaultCapable(false);
    exitButton.setText("Exit");
    exitButton.setActionCommand("Exit");

    JToolBar1.add(defaultConnectBtn);
    JToolBar1.add(connectLocalButton);
    JToolBar1.add(connectButton);
    JToolBar1.add(disconnectButton);
    JToolBar1.add(metricButton);
    JToolBar1.add(legendButton);
    JToolBar1.add(JToolBarSeparator1);
    JToolBar1.add(JToolBarSeparator2);
    JToolBar1.add(exitButton);

    ConnectionMenu.setText("Connection");
    ConnectionMenu.setActionCommand("Connection");
    ConnectionMenu.setMnemonic((int)'C');

    ConnectMenu.setRequestFocusEnabled(true);
    ConnectMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    ConnectMenu.setText("Connect");
    ConnectMenu.setActionCommand("Connect");
    ConnectMenu.setMnemonic((int)'C');

    ConnectLocalMenu.setRequestFocusEnabled(true);
    ConnectLocalMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    ConnectLocalMenu.setText("Connect Localhost");
    ConnectLocalMenu.setActionCommand("Connect LocalHost");
    ConnectLocalMenu.setMnemonic((int)'L');

    DisconnectMenu.setRequestFocusEnabled(true);
    DisconnectMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    DisconnectMenu.setText("Disconnect");
    DisconnectMenu.setActionCommand("Disconnect");
    DisconnectMenu.setMnemonic((int)'D');

    TraceFileMenu.setRequestFocusEnabled(true);
    TraceFileMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    TraceFileMenu.setText("Open Trace File");
    TraceFileMenu.setActionCommand("TraceFile");
    TraceFileMenu.setMnemonic((int)'T');

    ExitMenu.setRequestFocusEnabled(true);
    ExitMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    ExitMenu.setText("Exit");
    ExitMenu.setActionCommand("Exit");
    ExitMenu.setMnemonic((int)'X');

    ConnectionMenu.add(ConnectMenu);
    ConnectionMenu.add(ConnectLocalMenu);
    ConnectionMenu.add(DisconnectMenu);
    ConnectionMenu.add(TraceFileMenu);
    ConnectionMenu.add(ExitMenu);

    OptionsMenu.setText("Options");
    OptionsMenu.setActionCommand("Options");

    pollingIntervalMenu.setRequestFocusEnabled(true);
    pollingIntervalMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    pollingIntervalMenu.setText("Polling Interval");
    pollingIntervalMenu.setActionCommand("Polling Interval");
    pollingIntervalMenu.setMnemonic((int)'P');

    alarmsMenu.setRequestFocusEnabled(true);
    alarmsMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    alarmsMenu.setText("Set Alarms");
    alarmsMenu.setActionCommand("Set Alarms");
    alarmsMenu.setMnemonic((int)'A');

    logLinearScaleMenu.setEnabled(false);
    logLinearScaleMenu.setRequestFocusEnabled(true);
    logLinearScaleMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    logLinearScaleMenu.setText("Log/Linear Scale");
    logLinearScaleMenu.setVisible(false);
    logLinearScaleMenu.setActionCommand("Log/Linear Scale");
    logLinearScaleMenu.setMnemonic((int)'L');

    lockScalesMenu.setRequestFocusEnabled(true);
    lockScalesMenu.setHorizontalTextPosition(SwingConstants.LEFT);
    lockScalesMenu.setText("Use Same Scale for All Graphs");
    lockScalesMenu.setActionCommand("LockScale");
    lockScalesMenu.setSelected(true);
    lockScalesMenu.setMnemonic((int)'S');

    OptionsMenu.add(lockScalesMenu);
    OptionsMenu.add(pollingIntervalMenu);
    OptionsMenu.add(alarmsMenu);
    //OptionsMenu.add(logLinearScaleMenu);

    HelpMenu.setText("Help");
    HelpMenu.setVisible(true);
    HelpMenu.setActionCommand("Help");

    aboutMenuItem.setEnabled(true);
    aboutMenuItem.setRequestFocusEnabled(true);
    aboutMenuItem.setHorizontalTextPosition(SwingConstants.LEFT);
    aboutMenuItem.setText("About Perfometer");
    aboutMenuItem.setVisible(true);
    aboutMenuItem.setActionCommand("About Perfometer");
    aboutMenuItem.setMnemonic((int)'A');

    helpMenuItem.setEnabled(true);
    helpMenuItem.setRequestFocusEnabled(true);
    helpMenuItem.setHorizontalTextPosition(SwingConstants.LEFT);
    helpMenuItem.setText("Perfometer Help");
    helpMenuItem.setVisible(true);
    helpMenuItem.setActionCommand("Perfometer Help");
    helpMenuItem.setMnemonic((int)'H');

    HelpMenu.add(aboutMenuItem);
    HelpMenu.add(helpMenuItem);

    MainMenu.add(ConnectionMenu);
    MainMenu.add(OptionsMenu);
    MainMenu.add(HelpMenu);

    connectButton.setIcon(connectIcon);
    exitButton.setIcon(exitIcon);
    defaultConnectBtn.setIcon(connectLocalIcon);
    connectLocalButton.setIcon(connectLocalIcon);
    metricButton.setIcon(metricIcon);
    legendButton.setIcon(legendIcon);
    disconnectButton.setIcon(disconnectIcon);
    
    SymAction lSymAction = new SymAction();

    ConnectMenu.addActionListener(lSymAction);
    ConnectLocalMenu.addActionListener(lSymAction);
    DisconnectMenu.addActionListener(lSymAction);
    pollingIntervalMenu.addActionListener(lSymAction);
    logLinearScaleMenu.addActionListener(lSymAction);
    lockScalesMenu.addActionListener(lSymAction);
    alarmsMenu.addActionListener(lSymAction);
    ExitMenu.addActionListener(lSymAction);
    TraceFileMenu.addActionListener(lSymAction);
    aboutMenuItem.addActionListener(lSymAction);
    helpMenuItem.addActionListener(lSymAction);

    connectButton.addActionListener(lSymAction);
    connectLocalButton.addActionListener(lSymAction);
    disconnectButton.addActionListener(lSymAction);
    exitButton.addActionListener(lSymAction);
    metricButton.addActionListener(lSymAction);
    legendButton.addActionListener(lSymAction);
    defaultConnectBtn.addActionListener(lSymAction);

    miniGraphs.setSize(getSize().width, 100);
    miniGraphs.setVisible(true);
    tabbedConnPane.setSize(getSize().width, 400);
    tabbedConnPane.setVisible(true);

    PerfDesktop.setTopComponent(tabbedConnPane);
    PerfDesktop.setBottomComponent(miniGraphs);
    PerfDesktop.setDividerLocation(400);

    mgMouseListener = new MiniGraphMouseListener();

    PerfMenuListener fl = new PerfMenuListener();

    ConnectionMenu.addMenuListener(fl);
    OptionsMenu.addMenuListener(fl);
    HelpMenu.addMenuListener(fl);

    getContentPane().add(BorderLayout.CENTER,PerfDesktop);

    this.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
            Perfometer.errLog("WindowClosing event");
            exitPerfometer(0);
        }
    });

    connectionInputBox = new InputBox(this, "Connection Info", "asdf",
       highLevelMetrics);
  }

  void exitPerfometer(int status) {
    if(startedFromCmdLine)
      System.exit(status); 
    else
      setVisible(false);
  }

  public void setDrawingDelay(int speed) {
    if(graphingThread != null)
      graphingThread.setDrawingDelay(speed);
  }

  class PerfMenuListener implements MenuListener {
    public void menuSelected(MenuEvent evt) {
      if(graphingThread != null)
        graphingThread.setDrawingDelay(GraphUpdateThread.VERY_SLOW);
    }
    public void menuDeselected(MenuEvent evt) {
      if(graphingThread != null)
        graphingThread.setDrawingDelay(GraphUpdateThread.FAST);
    }
    public void menuCanceled(MenuEvent evt) {
      if(graphingThread != null)
        graphingThread.setDrawingDelay(GraphUpdateThread.FAST);
    }
  }

  class MiniGraphMouseListener extends MouseAdapter {
    public void mouseClicked(java.awt.event.MouseEvent e) {
      if(tabbedConnPane.getTabCount() > 0) {
        try {
          PerfHist ph = (PerfHist) e.getComponent();
          tabbedConnPane.setSelectedComponent((Component) procmgr.get(ph.getUniqueID()));
        } catch (Exception exc) {
          Perfometer.errLog("Error switching tabs:  " + exc.getMessage());
        }
      }
    }
  }

  public PanelManager getPanelManager() {
    return procmgr;
  }

  public void alarmBreached(PerfBreachEvent evt)
  {
    PerfPanel src = (PerfPanel) procmgr.get(evt.getRank());

    if(graphingThread != null)
      graphingThread.setDrawingDelay(GraphUpdateThread.VERY_SLOW);

    AlarmBreach i = new AlarmBreach(connection, src, evt);

    if(graphingThread != null)
      graphingThread.setDrawingDelay(GraphUpdateThread.FAST);

    i.setLocation(50, 50);
    i.show();
  }

  public static void errLog(String message)
  {
    if(perfdebug) System.err.println(message);
  }

  public Perfometer(String sTitle)
  {
    this();
    setTitle(sTitle);
  }

  public void setVisible(boolean b)
  {
    if (b)
      setLocation(50, 50);
    super.setVisible(b);
  }

  public boolean scalesLocked() {
    return lockScalesMenu.isSelected();
  }

  static public void main(String args[])
  {
    // Get command line arguments
    if (args.length > 1) {
      if (args[0].toString().compareTo("-n") == 0) {
        try {
          histBars = Integer.valueOf(args[1]).intValue();
        } catch(Throwable t) {
            Perfometer.errLog(t.getMessage());
            System.exit(1);
        }
      }
    }

    perfdebug = System.getProperty("perfometer.debug", "false").equalsIgnoreCase("true");

    Perfometer p = Perfometer.getInstance();
    p.startedFromCmdLine = true;
    p.setVisible(true);
  }

  private static Perfometer perfInstance = null;

  public static Perfometer getInstance() {
    if(perfInstance == null)
       perfInstance = new Perfometer();
    return perfInstance;
  }

  public void addNotify()
  {
    // Record the size of the window prior to calling parents addNotify.
    Dimension size = getSize();

    super.addNotify();

    if (frameSizeAdjusted)
      return;

    frameSizeAdjusted = true;

    // Adjust size of frame according to the insets and menu bar
    Insets insets = getInsets();
    JMenuBar menuBar = getRootPane().getJMenuBar();

    int menuBarHeight = 0; 

    if (menuBar != null)
      menuBarHeight = menuBar.getPreferredSize().height;

    setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height + menuBarHeight);
  }

  class SymAction implements java.awt.event.ActionListener
  {

    public void actionPerformed(java.awt.event.ActionEvent event)
    {

      if(graphingThread != null)
        graphingThread.setDrawingDelay(GraphUpdateThread.VERY_SLOW);

      Object object = event.getSource();
      if (object == ConnectMenu)
        ConnectMenu_actionPerformed(event);
      else if (object == ConnectLocalMenu)
        ConnectLocalMenu_actionPerformed(event);
      else if (object == DisconnectMenu)
        DisconnectMenu_actionPerformed(event);
      else if (object == ExitMenu)
        ExitMenu_actionPerformed(event);
      else if (object == TraceFileMenu)
        TraceFileMenu_actionPerformed(event);
      else if (object == pollingIntervalMenu)
        pollingIntervalMenu_actionPerformed(event);
      else if (object == aboutMenuItem)
        helpAbout_actionPerformed(event);
      else if (object == helpMenuItem)
        perfometerHelp_actionPerformed(event);
      else if (object == lockScalesMenu)
        lockScalesMenu_actionPerformed(event);
      else if (object == logLinearScaleMenu)
        logLinearScaleMenu_actionPerformed(event);
      else if (object == alarmsMenu)
        alarmsMenu_actionPerformed(event);
      else if (object == connectButton)
        ConnectMenu_actionPerformed(event);
      else if (object == connectLocalButton)
        ConnectLocalMenu_actionPerformed(event);
      else if (object == disconnectButton)
        DisconnectMenu_actionPerformed(event);
      else if (object == exitButton)
        ExitMenu_actionPerformed(event);
      else if (object == metricButton)
        metricButton_actionPerformed(event);
      else if (object == legendButton)
        legendButton_actionPerformed(event);
      else if (object == defaultConnectBtn)
        defaultConnectBtn_actionPerformed(event);

      if(graphingThread != null)
        graphingThread.setDrawingDelay(GraphUpdateThread.FAST);
    }
  }

  void alarmsMenu_actionPerformed(java.awt.event.ActionEvent event)
  {
    Component c = tabbedConnPane.getSelectedComponent();

    if ( c instanceof PerfPanel) {
      PerfPanel p = (PerfPanel)c;

      AlarmInput i = new AlarmInput(p.getGraphLabel(), 
           p.getAlarmSettings());

      i.setLocation(getX() + 30, getY() + 30);
      i.show();

      AlarmSettings s = i.getSettings();

      if(s != null) {
        if(i.setAll())
          setAllAlarms(s);
        else
          p.setAlarmSettings(s);
      }

      return;
    } else {
      JOptionPane.showMessageDialog(this, "You are not connected.");
    }

    return;
  }

  public static int parsePortString(String s) {
    java.util.StringTokenizer st = new java.util.StringTokenizer(s);
    int portNum = -1;

    while(st.hasMoreTokens()) {
			String tok = st.nextToken();
      try {
        portNum = Integer.parseInt(tok.substring(1, tok.length()-1));
        break;
      }catch(Exception e) {
        // continue...
      }
    }

    return portNum;
  }

  public void setAllAlarms(AlarmSettings s) {
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      perfpanel.setAlarmSettings((AlarmSettings)s.clone());
    }
  }

  static String[] aboutString = { "Perfometer Beta 0.9", "Jan 2001"};

  void helpAbout_actionPerformed(java.awt.event.ActionEvent event)
  {
    JOptionPane.showMessageDialog(this, aboutString, 
       "About Perfometer", JOptionPane.INFORMATION_MESSAGE, largeICLIcon);
    return;
  }

  void perfometerHelp_actionPerformed(java.awt.event.ActionEvent event)
  {
    JOptionPane.showMessageDialog(this, "Help coming soon..");
    return;
  }

  void ConnectMenu_actionPerformed(java.awt.event.ActionEvent event)
  {
    establishConnection(false);
  }

  void ConnectLocalMenu_actionPerformed(java.awt.event.ActionEvent event)
  {
    establishConnection(true);
  }

  void defaultConnectBtn_actionPerformed(java.awt.event.ActionEvent event)
  {
    establishConnection("localhost", "localhost", 3545, true, true);
  }

  public void establishConnection(boolean isLocal) {
    establishConnection(null, null, -1, isLocal, false);
  }

  public void establishConnection(String host, int port) {
    establishConnection(host, host, port, false, true);
  }

  int queryDisconnect() {
    int r = JOptionPane.showConfirmDialog(this,
          "Already connected -- Close current connection?", 
          "Disconnect current?", JOptionPane.YES_NO_OPTION);

    if (r == JOptionPane.YES_OPTION)
      disconnect();

    return r;
  }

  public void establishConnection(String connectTo, String host, int portNum,
    boolean isLocal, boolean noDialog)
  {
    MetricParser metric = new MflopParser();

    if(connection != null)
      if(queryDisconnect() != JOptionPane.YES_OPTION)
         return;

    if(!noDialog) {
      if(isLocal)
        prevHost = "localhost";

      connectionInputBox.setHostname(prevHost);
      connectionInputBox.setLocation(this.getX() + 30, this.getY() + 30);
      connectionInputBox.show();

      connectTo = connectionInputBox.getHostname();
      metric = connectionInputBox.getSelectedMetric();

      if((connectTo != null) && !connectTo.equals(""))
        prevHost = connectionInputBox.hostname;

      portNum = connectionInputBox.portNum;
    }

    if (connectTo == null || portNum == -1) return;

    StringTokenizer st = new StringTokenizer(connectTo, ".");
    host = (String)st.nextElement();

    try {
      procmgr = new PanelManager();

      connection = new LineReader(procmgr, metric);

      ConnectionEstablishmentDialog cd = 
         new ConnectionEstablishmentDialog(this, connection, connectTo, portNum);

      connection.setAppInput(cd);
      connection.addBreachListener(this);

      cd.startConnecting();

      int connStatus = cd.getStatus();

      switch(connStatus) {
        case ConnectionEstablishmentDialog.CONN_NORMAL:
          // normal connection.. do nothing here
          break;
        case ConnectionEstablishmentDialog.CONN_ABNORMAL:
          throw new Exception(cd.perror());
        case ConnectionEstablishmentDialog.CONN_INTERRUPTED:
          throw new InterruptedException("user interrupted");
      }

      sendAllMetrics(metric);
      connection.setPriority(CONNECTION_PRIORITY);

      graphingThread = new GraphUpdateThread(connection, procmgr);
      graphingThread.setPriority(GRAPHING_PRIORITY);

      setupPanels(connectTo);

      connection.start();
      graphingThread.start();

      if(connection.isServer())
        alarmsMenu.setEnabled(true);
      else
        alarmsMenu.setEnabled(false);
    } catch (Exception e){

       if(connection != null) {
         connection.disconnect();
         connection = null;
         if(graphingThread != null) {
           graphingThread.stopGraphing();
           graphingThread = null;
         } 
       }

       if(! (e instanceof InterruptedException)) {
         String reason = e.getMessage();

         if(reason == null)
           reason = "Reason Unknown";

         JOptionPane.showMessageDialog(this, "Connection failed: " + 
           reason);
       }

       return;
    }
  }

  void setupPanels(String hostname) throws IOException {
    PerfPanel perfpanel = null;
    boolean useMini = (procmgr.numElements() > 1);

    if(useMini)
      PerfDesktop.setDividerLocation(400);  // use absolute location
    else
      PerfDesktop.setDividerLocation(1.0);  // use proportional location

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();

      perfpanel.setSize(tabbedConnPane.getSize());
      tabbedConnPane.addTab(perfpanel.getGraphLabel(), perfpanel);

      if(useMini) {
        miniGraphs.add(perfpanel.getMiniGraph());
        perfpanel.getMiniGraph().addMouseListener(mgMouseListener);
      }
    }
  }

  void DisconnectMenu_actionPerformed(java.awt.event.ActionEvent event)
  {
    disconnect();
  }
 
  void disconnect() {
    tabbedConnPane.removeAll();
    tabbedConnPane.repaint();
    miniGraphs.removeAll();
    miniGraphs.repaint();

    if(connection != null) {
      connection.disconnect();
      connection = null;
      graphingThread.stopGraphing();
      graphingThread = null;
    }
  }

  class ErrorMessage extends JFrame 
    implements java.awt.event.ActionListener
  {
    JLabel label;
    JButton okButton;

    public ErrorMessage(String title, String text) {
      this.setTitle(title);
      this.getContentPane().setLayout(new BorderLayout(0,0));
      this.setSize(text.length() * 8, 100);
      this.setVisible(false);

      label = new JLabel(text);
      okButton = new JButton("OK");
      okButton.addActionListener(this);

      JPanel p = new JPanel();
      p.setLayout(new FlowLayout(FlowLayout.CENTER));
      p.add(okButton);

      this.getContentPane().add(BorderLayout.NORTH,label);
      this.getContentPane().add(BorderLayout.CENTER,p);
    }

    public void actionPerformed(java.awt.event.ActionEvent e) {
      if(e.getSource() == okButton)
        this.dispose();
    }
  }

  void TraceFileMenu_actionPerformed(java.awt.event.ActionEvent event)
  {
    MetricParser metric = new MflopParser();
    File f;

    if(connection != null)
      if(queryDisconnect() != JOptionPane.YES_OPTION)
         return;

    try {
      fc.setCurrentDirectory(new File(System.getProperty("user.dir")));
    }catch(Exception e) { }

    fc.setMultiSelectionEnabled(false);

    if(fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {

      f = fc.getSelectedFile();

      try {
        procmgr = new PanelManager();
        connection = new TraceFileReader(f, procmgr);

        ((TraceFileReader)connection).establishConnection();

        sendAllMetrics(metric);
        connection.setPriority(CONNECTION_PRIORITY);

        graphingThread = new GraphUpdateThread(connection, procmgr);
        graphingThread.setPriority(GRAPHING_PRIORITY);

        setupPanels(f.getCanonicalPath());

        connection.start();
        graphingThread.start();
      } catch (Exception e){
         if(connection != null) {
           connection.disconnect();
           connection = null;
         }

         JOptionPane.showMessageDialog(this, "Connection failed: " +
            e.getMessage());
         return;
      }
    }
  }

  void ExitMenu_actionPerformed(java.awt.event.ActionEvent event)
  {
    exitPerfometer(0);
  }

  void pollingIntervalMenu_actionPerformed(ActionEvent event)
  {
    Component c = tabbedConnPane.getSelectedComponent();

    if ( c instanceof PerfPanel) {
      PerfPanel p = (PerfPanel)c;

      // Set the microseconds portion of the polling interval
      String s = (String)JOptionPane.showInputDialog(this,
        "Enter the polling interval ms (current value=" + 
        p.getPollingIntervalms() + "):", 
        "Enter the polling interval ms:", JOptionPane.QUESTION_MESSAGE,
        null,null, null);

      if (s != null) {
        int ms = new Integer(s).intValue();
        p.setPollingIntervalms(ms);
      }

      return;
    } else {
      JOptionPane.showMessageDialog(this,
          "Please click on a tab with an active connection first.");
    }
  }

  void lockScalesMenu_actionPerformed(ActionEvent event)
  {
    // do nothing here (for now)
  }

  void logLinearScaleMenu_actionPerformed(ActionEvent event)
  {
    Component c = tabbedConnPane.getSelectedComponent();

    if ( c instanceof PerfPanel) {
      PerfPanel p = (PerfPanel)c;

      Object[] options = { "Log Scale", "Linear Scale" }; 
      int n = JOptionPane.showOptionDialog(this, "Select Log or Linear scale", "Perfometer",
      JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[0]); 

      p.setLogLinearScale(n);

      return;
    } else {
      JOptionPane.showMessageDialog(this,
         "Please click on a tab with an active connection first.");
    }
  }

  void legendButton_actionPerformed(java.awt.event.ActionEvent event)
  {
    Component c = tabbedConnPane.getSelectedComponent();
    PerfProcessInfo procinfo = null;

    if ( c instanceof PerfPanel) {
      PerfPanel p = (PerfPanel)c;

      MetricParser mp = p.getMetricParser();

      LegendFrame lf = new LegendFrame(mp);
      lf.setVisible(true);
    }
  }

  void metricButton_actionPerformed(java.awt.event.ActionEvent event)
  {
    Component c = tabbedConnPane.getSelectedComponent();
    PerfProcessInfo procinfo = null;

    if ( c instanceof PerfPanel) {

      PerfPanel p = (PerfPanel)c;

      procinfo = p.getProcInfo();

      pauseAllGraphs(true);

      MetricSelectionDialog md = new MetricSelectionDialog(highLevelMetrics, p.getMetrics());

      md.setLocation(this.getX() + 30, this.getY() + 30);
      md.show();

      MetricParser metric = md.getSelectedMetric();
 
      if(metric != null) {
        try {
          clearAllGraphs();
          connection.sendChangeMetric(p.getRank(), metric);

          if(procinfo != null) {
            MetricParser mp = procmgr.getMetricParser(p.getRank());
            mp.setProcInfo(procinfo);
          }
          
          relabelAllGraphs();
        }catch(UnsupportedMetricException e) {
          JOptionPane.showMessageDialog(this, e.getMessage());
        }
      }

      pauseAllGraphs(false);

    } else {
      JOptionPane.showMessageDialog(this,
          "Please click on a tab with an active connection first.");
    }
  }

  public void pauseAllGraphs(boolean s) {
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      perfpanel.setPaused(s);
    }
  }

  public void sendAllMetrics(MetricParser p) 
   throws UnsupportedMetricException
  {
    PerfProcessInfo procinfo;
    PerfPanel perfpanel;

errLog("in sendallmetrics...");

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      procinfo = perfpanel.getProcInfo();

errLog("sending metric " + p);
      connection.sendChangeMetricUnsynchronized(perfpanel.getRank(), p);

      if(procinfo != null) {
        MetricParser mp = procmgr.getMetricParser(perfpanel.getRank());
        mp.setProcInfo(procinfo);
      }
    }
  }

  public void clearAllGraphs() {
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      perfpanel.clearPoints();
    }
  }

  public void relabelAllGraphs() {
    PerfPanel perfpanel;

    for (Enumeration e = procmgr.elements() ; e.hasMoreElements() ;) {
      perfpanel = (PerfPanel) e.nextElement();
      perfpanel.initDisplay();
    }
  }

}
